Package: graphlayouts
Title: Additional Layout Algorithms for Network Visualizations
Version: 1.2.4
Authors@R: person("David", "Schoch", email = "david@schochastics.net", role = c("aut", "cre"),
           comment = c(ORCID = "0000-0003-2952-4812"))
Description: Several new layout algorithms to visualize networks are provided which are not part of 'igraph'. 
    Most are based on the concept of stress majorization by Gansner et al. (2004) <doi:10.1007/978-3-540-31843-9_25>. 
    Some more specific algorithms allow the user to emphasize hidden group structures in networks or focus on specific nodes.
URL: https://github.com/schochastics/graphlayouts,
        https://schochastics.github.io/graphlayouts/
BugReports: https://github.com/schochastics/graphlayouts/issues
Depends: R (>= 3.6.0)
License: MIT + file LICENSE
Encoding: UTF-8
LazyData: true
Imports: igraph (>= 2.0.0), Rcpp
Suggests: testthat, ggplot2, uwot, oaqc
LinkingTo: Rcpp, RcppArmadillo
RoxygenNote: 7.3.2
NeedsCompilation: yes
Packaged: 2026-06-18 18:23:41 UTC; david
Author: David Schoch [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2952-4812>)
Maintainer: David Schoch <david@schochastics.net>
Repository: CRAN
Date/Publication: 2026-06-19 05:10:17 UTC
Built: R 4.5.2; x86_64-apple-darwin20; 2026-06-19 07:08:09 UTC; unix
Archs: graphlayouts.so.dSYM

# Generated by roxygen2: do not edit by hand

export(annotate_circle)
export(draw_circle)
export(layout_as_backbone)
export(layout_as_dynamic)
export(layout_as_metromap)
export(layout_as_multilevel)
export(layout_igraph_backbone)
export(layout_igraph_centrality)
export(layout_igraph_centrality_group)
export(layout_igraph_constrained_stress)
export(layout_igraph_eigen)
export(layout_igraph_fixed_coords)
export(layout_igraph_focus)
export(layout_igraph_focus_group)
export(layout_igraph_multilevel)
export(layout_igraph_pmds)
export(layout_igraph_sparse_stress)
export(layout_igraph_stress)
export(layout_igraph_umap)
export(layout_mirror)
export(layout_rotate)
export(layout_with_centrality)
export(layout_with_centrality_group)
export(layout_with_constrained_stress)
export(layout_with_constrained_stress3D)
export(layout_with_eigen)
export(layout_with_fixed_coords)
export(layout_with_focus)
export(layout_with_focus_group)
export(layout_with_pmds)
export(layout_with_sparse_stress)
export(layout_with_stress)
export(layout_with_stress3D)
export(layout_with_umap)
export(reorder_edges)
importFrom(Rcpp,sourceCpp)
useDynLib(graphlayouts, .registration = TRUE)

<!-- README.md is generated from README.Rmd. Please edit that file -->

# graphlayouts <img src="man/figures/logo.png" align="right"/>

[![R-CMD-check](https://github.com/schochastics/graphlayouts/workflows/R-CMD-check/badge.svg)](https://github.com/schochastics/graphlayouts/actions)
[![CRAN
status](https://www.r-pkg.org/badges/version/graphlayouts)](https://cran.r-project.org/package=graphlayouts)
[![Downloads](https://cranlogs.r-pkg.org/badges/graphlayouts)](https://CRAN.R-project.org/package=graphlayouts)
[![Total
Downloads](https://cranlogs.r-pkg.org/badges/grand-total/graphlayouts)](https://CRAN.R-project.org/package=graphlayouts)
[![Codecov test
coverage](https://codecov.io/gh/schochastics/graphlayouts/branch/main/graph/badge.svg)](https://app.codecov.io/gh/schochastics/graphlayouts?branch=main)
[![Zenodo](https://zenodo.org/badge/DOI/10.5281/zenodo.7870213.svg)](https://doi.org/10.5281/zenodo.7870213)
[![JOSS](https://joss.theoj.org/papers/10.21105/joss.05238/status.svg)](https://doi.org/10.21105/joss.05238)

This package implements some graph layout algorithms that are not
available in `igraph`.

**A detailed introductory tutorial for graphlayouts and ggraph can be
found [here](https://schochastics.github.io/netVizR/).**

The package implements the following algorithms:

- Stress majorization
  ([Paper](https://graphviz.gitlab.io/_pages/Documentation/GKN04.pdf))
- Quadrilateral backbone layout
  ([Paper](https://doi.org/10.7155/jgaa.00370))
- flexible radial layouts
  ([Paper](https://www.uni-konstanz.de/algo/publications/bp-mfrl-11.pdf))
- sparse stress ([Paper](https://arxiv.org/abs/1608.08909))
- pivot MDS ([Paper](https://doi.org/10.1007/978-3-540-70904-6_6))
- dynamic layout for longitudinal data
  ([Paper](https://doi.org/10.1016/j.socnet.2011.06.002))
- spectral layouts (adjacency/Laplacian)
- a simple multilevel layout
- a layout algorithm using UMAP
- group based centrality and focus layouts which keeps groups of nodes
  close in the same range on the concentric circle

## Install

``` r
# dev version
remotes::install_github("schochastics/graphlayouts")

# CRAN
install.packages("graphlayouts")
```

## Stress Majorization: Connected Network

*This example is a bit of a special case since it exploits some weird
issues in igraph.*

``` r
library(igraph)
library(ggraph)
library(graphlayouts)

set.seed(666)
pa <- sample_pa(1000, 1, 1, directed = F)

ggraph(pa, layout = "nicely") +
    geom_edge_link0(width = 0.2, colour = "grey") +
    geom_node_point(col = "black", size = 0.3) +
    theme_graph()
```

<img src="man/figures/README-example-1.png" width="80%" style="display: block; margin: auto;" />

``` r

ggraph(pa, layout = "stress") +
    geom_edge_link0(width = 0.2, colour = "grey") +
    geom_node_point(col = "black", size = 0.3) +
    theme_graph()
```

<img src="man/figures/README-example-2.png" width="80%" style="display: block; margin: auto;" />

## Stress Majorization: Unconnected Network

Stress majorization also works for networks with several components. It
relies on a bin packing algorithm to efficiently put the components in a
rectangle, rather than a circle.

``` r
set.seed(666)
g <- disjoint_union(
    sample_pa(10, directed = FALSE),
    sample_pa(20, directed = FALSE),
    sample_pa(30, directed = FALSE),
    sample_pa(40, directed = FALSE),
    sample_pa(50, directed = FALSE),
    sample_pa(60, directed = FALSE),
    sample_pa(80, directed = FALSE)
)

ggraph(g, layout = "nicely") +
    geom_edge_link0() +
    geom_node_point() +
    theme_graph()
```

<img src="man/figures/README-example_un-1.png" width="80%" style="display: block; margin: auto;" />

``` r

ggraph(g, layout = "stress", bbox = 40) +
    geom_edge_link0() +
    geom_node_point() +
    theme_graph()
```

<img src="man/figures/README-example_un-2.png" width="80%" style="display: block; margin: auto;" />

## Backbone Layout

Backbone layouts are helpful for drawing hairballs.

``` r
set.seed(665)
# create network with a group structure
g <- sample_islands(9, 40, 0.4, 15)
g <- simplify(g)
V(g)$grp <- as.character(rep(1:9, each = 40))

ggraph(g, layout = "stress") +
    geom_edge_link0(colour = rgb(0, 0, 0, 0.5), width = 0.1) +
    geom_node_point(aes(col = grp)) +
    scale_color_brewer(palette = "Set1") +
    theme_graph() +
    theme(legend.position = "none")
```

<img src="man/figures/README-hairball-1.png" width="80%" style="display: block; margin: auto;" />

The backbone layout helps to uncover potential group structures based on
edge embeddedness and puts more emphasis on this structure in the
layout.

``` r
bb <- layout_as_backbone(g, keep = 0.4)
E(g)$col <- FALSE
E(g)$col[bb$backbone] <- TRUE

ggraph(g, layout = "manual", x = bb$xy[, 1], y = bb$xy[, 2]) +
    geom_edge_link0(aes(col = col), width = 0.1) +
    geom_node_point(aes(col = grp)) +
    scale_color_brewer(palette = "Set1") +
    scale_edge_color_manual(values = c(rgb(0, 0, 0, 0.3), rgb(0, 0, 0, 1))) +
    theme_graph() +
    theme(legend.position = "none")
```

<img src="man/figures/README-backbone-1.png" width="80%" style="display: block; margin: auto;" />

## Radial Layout with Focal Node

The function `layout_with_focus()` creates a radial layout around a
focal node. All nodes with the same distance from the focal node are on
the same circle.

``` r
library(igraphdata)
library(patchwork)
data("karate")

p1 <- ggraph(karate, layout = "focus", focus = 1) +
    draw_circle(use = "focus", max.circle = 3) +
    geom_edge_link0(edge_color = "black", edge_width = 0.3) +
    geom_node_point(aes(fill = as.factor(Faction)), size = 2, shape = 21) +
    scale_fill_manual(values = c("#8B2323", "#EEAD0E")) +
    theme_graph() +
    theme(legend.position = "none") +
    coord_fixed() +
    labs(title = "Focus on Mr. Hi")

p2 <- ggraph(karate, layout = "focus", focus = 34) +
    draw_circle(use = "focus", max.circle = 4) +
    geom_edge_link0(edge_color = "black", edge_width = 0.3) +
    geom_node_point(aes(fill = as.factor(Faction)), size = 2, shape = 21) +
    scale_fill_manual(values = c("#8B2323", "#EEAD0E")) +
    theme_graph() +
    theme(legend.position = "none") +
    coord_fixed() +
    labs(title = "Focus on John A.")

p1 + p2
```

<img src="man/figures/README-flex_focus-1.png" width="80%" style="display: block; margin: auto;" />

## Radial Centrality Layout

The function `layout_with_centrality` creates a radial layout around the
node with the highest centrality value. The further outside a node is,
the more peripheral it is.

``` r
library(igraphdata)
library(patchwork)
data("karate")

bc <- betweenness(karate)
p1 <- ggraph(karate, layout = "centrality", centrality = bc, tseq = seq(0, 1, 0.15)) +
    draw_circle(use = "cent") +
    annotate_circle(bc, format = "", pos = "bottom") +
    geom_edge_link0(edge_color = "black", edge_width = 0.3) +
    geom_node_point(aes(fill = as.factor(Faction)), size = 2, shape = 21) +
    scale_fill_manual(values = c("#8B2323", "#EEAD0E")) +
    theme_graph() +
    theme(legend.position = "none") +
    coord_fixed() +
    labs(title = "betweenness centrality")


cc <- closeness(karate)
p2 <- ggraph(karate, layout = "centrality", centrality = cc, tseq = seq(0, 1, 0.2)) +
    draw_circle(use = "cent") +
    annotate_circle(cc, format = "scientific", pos = "bottom") +
    geom_edge_link0(edge_color = "black", edge_width = 0.3) +
    geom_node_point(aes(fill = as.factor(Faction)), size = 2, shape = 21) +
    scale_fill_manual(values = c("#8B2323", "#EEAD0E")) +
    theme_graph() +
    theme(legend.position = "none") +
    coord_fixed() +
    labs(title = "closeness centrality")

p1 + p2
```

<img src="man/figures/README-flex_cent-1.png" width="80%" style="display: block; margin: auto;" />

## Large graphs

`graphlayouts` implements two algorithms for visualizing large networks
(\<100k nodes). `layout_with_pmds()` is similar to `layout_with_mds()`
but performs the multidimensional scaling only with a small number of
pivot nodes. Usually, 50-100 are enough to obtain similar results to the
full MDS.

`layout_with_sparse_stress()` performs stress majorization only with a
small number of pivots (~50-100). The runtime performance is inferior to
pivotMDS but the quality is far superior.

A comparison of runtimes and layout quality can be found in the
[wiki](https://github.com/schochastics/graphlayouts/wiki/)  
**tl;dr**: both layout algorithms appear to be faster than the fastest
igraph algorithm `layout_with_drl()`.

Below are two examples of layouts generated for large graphs using
`layout_with_sparse_stress()`

<img src="man/figures/rt-net.png" width="80%" style="display: block; margin: auto;" />
A retweet network with 18k nodes and 61k edges

<img src="man/figures/squad_network2022_small.png" width="80%" style="display: block; margin: auto;" />
A network of football players with 165K nodes and 6M edges.

## dynamic layouts

`layout_as_dynamic()` allows you to visualize snapshots of longitudinal
network data. Nodes are anchored with a reference layout and only moved
slightly in each wave depending on deleted/added edges. In this way, it
is easy to track down specific nodes throughout time. Use `patchwork` to
put the individual plots next to each other.

``` r
# remotes::install_github("schochastics/networkdata")
library(networkdata)
# longitudinal dataset of friendships in a school class
data("s50")

xy <- layout_as_dynamic(s50, alpha = 0.2)
pList <- vector("list", length(s50))

for (i in seq_along(s50)) {
    pList[[i]] <- ggraph(s50[[i]], layout = "manual", x = xy[[i]][, 1], y = xy[[i]][, 2]) +
        geom_edge_link0(edge_width = 0.6, edge_colour = "grey66") +
        geom_node_point(shape = 21, aes(fill = as.factor(smoke)), size = 3) +
        geom_node_text(aes(label = 1:50), repel = T) +
        scale_fill_manual(
            values = c("forestgreen", "grey25", "firebrick"),
            labels = c("no", "occasional", "regular"),
            name = "smoking",
            guide = ifelse(i != 2, "none", "legend")
        ) +
        theme_graph() +
        theme(legend.position = "bottom") +
        labs(title = paste0("Wave ", i))
}
wrap_plots(pList)
```

<img src="man/figures/dynamic_ex.png" width="80%" style="display: block; margin: auto;" />

## Layout manipulation

The functions `layout_mirror()` and `layout_rotate()` can be used to
manipulate an existing layout

<img src="man/figures/layout_manipulation.png" width="80%" style="display: block; margin: auto;" />

# How to reach out?

### Where do I report bugs?

Simply [open an
issue](https://github.com/schochastics/graphlayouts/issues/new) on
GitHub.

### How do I contribute to the package?

If you have an idea (but no code yet), [open an
issue](https://github.com/schochastics/graphlayouts/issues/new) on
GitHub. If you want to contribute with a specific feature and have the
code ready, fork the repository, add your code, and create a pull
request.

### Do you need support?

The easiest way is to [open an
issue](https://github.com/schochastics/graphlayouts/issues/new) - this
way, your question is also visible to others who may face similar
problems.

### Code of Conduct

Please note that the graphlayouts project is released with a
[Contributor Code of
Conduct](https://contributor-covenant.org/version/2/1/CODE_OF_CONDUCT.html).
By contributing to this project, you agree to abide by its terms.

#' Multilevel example Network
#' @description Multilevel network, where both levels have different structural features

#' @format igraph object
"multilvl_ex"

#' Subway network of Berlin
#'
#' A dataset containing the subway network of Berlin
#' @format igraph object
#' @references
#' Kujala, Rainer, et al. "A collection of public transport network data sets for 25 cities." Scientific data 5 (2018): 180089.
"metro_berlin"

#' @title manipulate layout
#' @description functions to manipulate an existing layout
#'
#' @param xy graph layout
#' @param angle angle for rotation
#' @param axis mirror horizontal or vertical
#' @name layout_manipulate
#' @details These functions are mostly useful for deterministic layouts such as [layout_with_stress]
#' @return manipulated matrix of xy coordinates
#' @examples
#' library(igraph)
#' g <- sample_gnp(50, 0.3)
#'
#' xy <- layout_with_stress(g)
#'
#' # rotate 90 degrees
#' xy <- layout_rotate(xy, 90)
#'
#' # flip horizontally
#' xy <- layout_mirror(xy, "horizontal")
#'
#' @author David Schoch
NULL

#' @rdname layout_manipulate
#' @export

layout_rotate <- function(xy, angle) {
  if (!"matrix" %in% class(xy)) {
    stop("xy must be a matrix")
  }
  if (!is.numeric(angle)) {
    stop("angle must be numeric")
  }
  radians <- angle * pi / 180
  s <- sin(radians)
  c <- cos(radians)

  cbind(xy[, 1] * c - xy[, 2] * s, xy[, 1] * s + xy[, 2] * c)
}

#' @rdname layout_manipulate
#' @export

layout_mirror <- function(xy, axis = "vertical") {
  if (!"matrix" %in% class(xy)) {
    stop("xy must be a matrix")
  }
  axis <- match.arg(axis, c("horizontal", "vertical"))
  if (axis == "horizontal") {
    middle <- mean(xy[, 2])
    ynew <- middle - (xy[, 2] - middle)
    xnew <- xy[, 1]
  } else if (axis == "vertical") {
    middle <- mean(xy[, 1])
    xnew <- middle - (xy[, 1] - middle)
    ynew <- xy[, 2]
  }
  xynew <- cbind(xnew, ynew)
  colnames(xynew) <- NULL
  xynew
}

test_that("layout_manipulate works", {
  xy <- cbind(c(0, 0, 0, 0), c(1, 2, 3, 4))
  xynew <- layout_mirror(
    layout_mirror(xy, axis = "horizontal"),
    axis = "horizontal"
  )
  expect_true(all(xynew == xy))
  xynew <- layout_mirror(
    layout_mirror(xy, axis = "vertical"),
    axis = "vertical"
  )
  expect_true(all(xynew == xy))

  expect_error(layout_mirror(xy, "diagonal"))
  expect_error(layout_mirror(c(1, 1), "horizontal"))

  expect_equal(layout_rotate(xy, 180)[, 2], c(-1, -2, -3, -4))
  expect_error(layout_rotate(c(1, 1), 70))
  expect_error(layout_rotate(xy, "70"))
})

#' Draw concentric circles
#'
#' @param col color of circles
#' @param use one of 'focus' or 'cent'
#' @param max.circle if use = 'focus' specifies the number of circles to draw
#' @details this function is best used with a concentric layout such as [layout_with_focus] and [layout_with_centrality].
#' @return concentric circles around origin
#' @examples
#' library(igraph)
#' g <- sample_gnp(10, 0.4)
#'
#' \dontrun{
#' library(ggraph)
#' ggraph(g, layout = "centrality", centrality = degree(g)) +
#'     draw_circle(use = "cent") +
#'     geom_edge_link() +
#'     geom_node_point(shape = 21, fill = "grey25", size = 5) +
#'     theme_graph() +
#'     coord_fixed()
#' }
#' @export
#'

draw_circle <- function(col = "#00BFFF", use = "focus", max.circle) {
    if (!requireNamespace("ggplot2", quietly = TRUE)) {
        stop("ggplot2 needed for this function to work. Please install it.", call. = FALSE)
    }
    if (!use %in% c("focus", "cent")) {
        stop("use must be one of 'focus' or 'cent'")
    }
    if (use == "focus" && missing(max.circle)) {
        stop("max.circle missing. Should be set to the max distance from focal node.")
    }
    dat <- data.frame()
    if (use == "cent") {
        for (d in seq(0, 100, 20) * 2) {
            tmp <- as.data.frame(circleFun(center = c(0, 0), diameter = d))
            tmp[["grp"]] <- d
            dat <- rbind(dat, tmp)
        }
    } else if (use == "focus") {
        for (d in 1:max.circle) {
            tmp <- as.data.frame(circleFun(center = c(0, 0), diameter = 2 * d))
            tmp[["grp"]] <- d
            dat <- rbind(dat, tmp)
        }
    }
    circs <- ggplot2::geom_path(data = dat, ggplot2::aes_(x = ~x, y = ~y, group = ~grp), col = col, alpha = 0.5)
    return(circs)
}


#' annotate concentric circles
#'
#' @param cent centrality scores used for layout
#' @param col color of text
#' @param format either empty string or 'scientific'
#' @param pos position of text ('top' or 'bottom')
#' @param text_size font size for annotations
#' @details this function is best used with [layout_with_centrality] together with [draw_circle].
#' @return annotated concentric circles around origin
#' @examples
#' library(igraph)
#'
#' g <- sample_gnp(10, 0.4)
#' \dontrun{
#' library(ggraph)
#' ggraph(g, layout = "centrality", centrality = closeness(g)) +
#'     draw_circle(use = "cent") +
#'     annotate_circle(closeness(g), pos = "bottom", format = "scientific") +
#'     geom_edge_link() +
#'     geom_node_point(shape = 21, fill = "grey25", size = 5) +
#'     theme_graph() +
#'     coord_fixed()
#' }
#' @export
#'
annotate_circle <- function(cent, col = "#00BFFF", format = "", pos = "top", text_size = 3) {
    if (!requireNamespace("ggplot2", quietly = TRUE)) {
        stop("ggplot2 needed for this function to work. Please install it.", call. = FALSE)
    }
    if (length(cent) == 1) {
        cent <- seq(1, cent, 1)
        dat_annot <- data.frame(y = seq(0, 100, 20), x = 0, val = interpolate_cent(cent, seq(0, 100, 20)))
        dat_annot[["val"]] <- round(dat_annot[["val"]], 8)
    } else {
        dat_annot <- data.frame(y = 100 - seq(0, 100, 20), x = 0, val = interpolate_cent(cent, seq(0, 100, 20)))
        dat_annot[["val"]] <- round(dat_annot[["val"]], 8)
    }
    vju <- 0
    if (format == "scientific") {
        dat_annot[["val"]] <- format(dat_annot[["val"]], scientific = TRUE)
    }
    if (pos == "bottom") {
        dat_annot[["y"]] <- -dat_annot[["y"]]
        vju <- 1
    }

    circs <- ggplot2::geom_text(
        data = dat_annot, ggplot2::aes_(x = ~x, y = ~y, label = ~val),
        col = col, vjust = vju, size = text_size
    )
    return(circs)
}


# helper -----
circleFun <- function(center = c(0, 0), diameter = 1, npoints = 100) {
    r <- diameter / 2
    tt <- seq(0, 2 * pi, length.out = npoints)
    xx <- center[1] + r * cos(tt)
    yy <- center[2] + r * sin(tt)
    return(data.frame(x = xx, y = yy))
}

interpolate_cent <- function(cent, x) {
    a <- min(cent)
    b <- max(cent)
    alpha <- 100 / (b - a)
    beta <- -100 / (b - a) * a
    (x - beta) / alpha
}

#' @title Manipulate graph
#' @description functions to manipulate a graph
#'
#' @param g igraph object
#' @param attr edge attribute name used to sort edges
#' @param desc logical. sort in descending (default) or ascending order
#' @details `reorder_edges()` allows to reorder edges according to an attribute so that edges are
#' drawn in the given order.
#' @name graph_manipulate
#' @return manipulated graph
#' @examples
#' library(igraph)
#'
#' g <- sample_gnp(10, 0.5)
#' E(g)$attr <- 1:ecount(g)

#' gn <- reorder_edges(g,"attr")
#'
#' @author David Schoch
NULL

#' @rdname graph_manipulate
#' @export

reorder_edges <- function(g, attr, desc = TRUE) {
    if (!"name" %in% igraph::vertex_attr_names(g)) {
        igraph::V(g)$name <- 1:igraph::vcount(g)
    }
    edges_df <- igraph::as_data_frame(g, what = "edges")
    edges_df <- edges_df[order(edges_df[[attr]], decreasing = desc), ]

    vertices <- igraph::as_data_frame(g, what = "vertices")
    vattrs <- igraph::vertex_attr_names(g)
    idname <- which(vattrs == "name")
    vertices <- vertices[, c(idname, setdiff(seq_along(vattrs), idname))]

    gn <- igraph::graph_from_data_frame(
        d = edges_df,
        directed = igraph::is_directed(g),
        vertices = vertices
    )
    gn
}

test_that("reorder_edges works", {
  g <- igraph::make_full_graph(3)
  igraph::E(g)$weight <- c(1, 2, 3)
  g1 <- graphlayouts::reorder_edges(g, "weight")
  expect_equal(igraph::E(g)$weight, rev(igraph::E(g1)$weight))
})

#' @rdname layout_stress
#' @param circular not used
#' @export
layout_igraph_stress <- function(
  g,
  weights = NA,
  iter = 500,
  tol = 0.0001,
  mds = TRUE,
  bbox = 30,
  circular
) {
  xy <- layout_with_stress(g, weights, iter, tol, mds, bbox)

  nodes <- data.frame(x = xy[, 1], y = xy[, 2])
  nodes$circular <- FALSE
  extraData <- as.data.frame(igraph::vertex_attr(g))
  nodes <- cbind(
    nodes,
    extraData[, !names(extraData) %in% names(nodes), drop = FALSE]
  )
  nodes
}

#' @rdname layout_focus
#' @param circular not used
#' @export
layout_igraph_focus <- function(
  g,
  v,
  weights = NA,
  iter = 500,
  tol = 0.0001,
  circular
) {
  xy <- layout_with_focus(g, v, weights, iter, tol)$xy
  nodes <- data.frame(x = xy[, 1], y = xy[, 2])
  nodes$circular <- FALSE
  extraData <- as.data.frame(igraph::vertex_attr(g))
  nodes <- cbind(
    nodes,
    extraData[, !names(extraData) %in% names(nodes), drop = FALSE]
  )
  nodes
}

#' @rdname layout_centrality
#' @param circular not used
#' @export
layout_igraph_centrality <- function(
  g,
  cent,
  scale = TRUE,
  iter = 500,
  tol = 0.0001,
  tseq = seq(0, 1, 0.2),
  circular
) {
  xy <- layout_with_centrality(g, cent, scale, iter, tol, tseq)
  nodes <- data.frame(x = xy[, 1], y = xy[, 2])
  nodes$circular <- FALSE
  extraData <- as.data.frame(igraph::vertex_attr(g))
  nodes <- cbind(
    nodes,
    extraData[, !names(extraData) %in% names(nodes), drop = FALSE]
  )
  nodes
}

#' @rdname layout_backbone
#' @param circular not used
#' @export
layout_igraph_backbone <- function(g, keep = 0.2, backbone = TRUE, circular) {
  xy <- layout_as_backbone(g, keep, backbone)$xy
  nodes <- data.frame(x = xy[, 1], y = xy[, 2])
  nodes$circular <- FALSE
  extraData <- as.data.frame(igraph::vertex_attr(g))
  nodes <- cbind(
    nodes,
    extraData[, !names(extraData) %in% names(nodes), drop = FALSE]
  )
  nodes
}

#' @rdname layout_spectral
#' @param circular not used
#' @export
layout_igraph_eigen <- function(
  g,
  type = "laplacian",
  ev = "smallest",
  circular
) {
  xy <- layout_with_eigen(g, type, ev)
  nodes <- data.frame(x = xy[, 1], y = xy[, 2])
  nodes$circular <- FALSE
  extraData <- as.data.frame(igraph::vertex_attr(g))
  nodes <- cbind(
    nodes,
    extraData[, !names(extraData) %in% names(nodes), drop = FALSE]
  )
  nodes
}

#' @rdname layout_pmds
#' @param circular not used
#' @export
layout_igraph_pmds <- function(g, pivots, weights = NA, D = NULL, circular) {
  xy <- layout_with_pmds(g, pivots, weights)
  nodes <- data.frame(x = xy[, 1], y = xy[, 2])
  nodes$circular <- FALSE
  extraData <- as.data.frame(igraph::vertex_attr(g))
  nodes <- cbind(
    nodes,
    extraData[, !names(extraData) %in% names(nodes), drop = FALSE]
  )
  nodes
}

#' @rdname layout_sparse_stress
#' @param circular not used
#' @export
layout_igraph_sparse_stress <- function(
  g,
  pivots,
  weights = NA,
  iter = 500,
  circular
) {
  xy <- layout_with_sparse_stress(g, pivots, weights, iter)
  nodes <- data.frame(x = xy[, 1], y = xy[, 2])
  nodes$circular <- FALSE
  extraData <- as.data.frame(igraph::vertex_attr(g))
  nodes <- cbind(
    nodes,
    extraData[, !names(extraData) %in% names(nodes), drop = FALSE]
  )
  nodes
}

#' @rdname layout_constrained_stress
#' @param circular not used
#' @export
layout_igraph_constrained_stress <- function(
  g,
  coord,
  fixdim = "x",
  weights = NA,
  iter = 500,
  tol = 0.0001,
  mds = TRUE,
  bbox = 30,
  circular
) {
  xy <- layout_with_constrained_stress(
    g,
    coord,
    fixdim,
    weights,
    iter,
    tol,
    mds,
    bbox
  )

  nodes <- data.frame(x = xy[, 1], y = xy[, 2])
  nodes$circular <- FALSE
  extraData <- as.data.frame(igraph::vertex_attr(g))
  nodes <- cbind(
    nodes,
    extraData[, !names(extraData) %in% names(nodes), drop = FALSE]
  )
  nodes
}

#' @rdname layout_multilevel
#' @param circular not used
#' @export
layout_igraph_multilevel <- function(
  g,
  type = "all",
  FUN1,
  FUN2,
  params1 = NULL,
  params2 = NULL,
  ignore_iso = TRUE,
  alpha = 35,
  beta = 45,
  circular
) {
  xy <- layout_as_multilevel(
    g,
    type,
    FUN1,
    FUN2,
    params1,
    params2,
    ignore_iso,
    alpha,
    beta
  )
  nodes <- data.frame(x = xy[, 1], y = xy[, 2])
  nodes$circular <- FALSE
  extraData <- as.data.frame(igraph::vertex_attr(g))
  nodes <- cbind(
    nodes,
    extraData[, !names(extraData) %in% names(nodes), drop = FALSE]
  )
  nodes
}

#' @rdname layout_umap
#' @param circular not used
#' @export
layout_igraph_umap <- function(g, circular, ...) {
  xy <- layout_with_umap(g, ...)
  nodes <- data.frame(x = xy[, 1], y = xy[, 2])
  nodes$circular <- FALSE
  extraData <- as.data.frame(igraph::vertex_attr(g))
  nodes <- cbind(
    nodes,
    extraData[, !names(extraData) %in% names(nodes), drop = FALSE]
  )
  nodes
}

#' @rdname layout_focus_group
#' @param circular not used
#' @export
layout_igraph_focus_group <- function(
  g,
  v,
  group,
  shrink = 10,
  weights = NA,
  iter = 500,
  tol = 0.0001,
  circular
) {
  xy <- layout_with_focus_group(g, v, group, shrink, weights, iter, tol)
  nodes <- data.frame(x = xy[, 1], y = xy[, 2])
  nodes$circular <- FALSE
  extraData <- as.data.frame(igraph::vertex_attr(g))
  nodes <- cbind(
    nodes,
    extraData[, !names(extraData) %in% names(nodes), drop = FALSE]
  )
  nodes
}

#' @rdname layout_centrality_group
#' @param circular not used
#' @export
layout_igraph_centrality_group <- function(
  g,
  cent,
  group,
  shrink = 10,
  circular,
  ...
) {
  xy <- layout_with_centrality_group(g, cent, group, shrink, ...)
  nodes <- data.frame(x = xy[, 1], y = xy[, 2])
  nodes$circular <- FALSE
  extraData <- as.data.frame(igraph::vertex_attr(g))
  nodes <- cbind(
    nodes,
    extraData[, !names(extraData) %in% names(nodes), drop = FALSE]
  )
  nodes
}


#' @rdname layout_fixed_coords
#' @param circular not used
#' @export
layout_igraph_fixed_coords <- function(
  g,
  coords,
  weights = NA,
  iter = 500,
  tol = 0.0001,
  mds = TRUE,
  bbox = 30,
  circular
) {
  xy <- layout_with_fixed_coords(g, coords, weights, iter, tol, mds, bbox)

  nodes <- data.frame(x = xy[, 1], y = xy[, 2])
  nodes$circular <- FALSE
  extraData <- as.data.frame(igraph::vertex_attr(g))
  nodes <- cbind(
    nodes,
    extraData[, !names(extraData) %in% names(nodes), drop = FALSE]
  )
  nodes
}

test_that("igraph function calls work", {
  testthat::skip_if_not_installed("oaqc")
  g <- igraph::make_full_graph(10,FALSE)
  igraph::V(g)$lvl <- rep(c(1,2),each = 5)
  expect_no_error(layout_igraph_centrality(g,cent = igraph::degree(g)))
  expect_no_error(layout_igraph_constrained_stress(g,fixdim="x",coord=rep(1,10)))
  expect_no_error(layout_igraph_eigen(g))
  expect_no_error(layout_igraph_focus(g,v = 1))
  expect_no_error(layout_igraph_multilevel(g))
  expect_no_error(layout_igraph_pmds(g,pivots = 5))
  expect_no_error(layout_igraph_sparse_stress(g,pivots = 5))
  expect_no_error(layout_igraph_stress(g))
  expect_no_error(layout_igraph_backbone(g))
})

ensure_igraph <- function(g) {
  if (!igraph::is_igraph(g)) {
    stop("g must be an igraph object", call. = FALSE)
  }
}

ensure_connected <- function(g) {
  if (!igraph::is_connected(g, mode = "weak")) {
    stop("only connected graphs are supported.", call. = FALSE)
  }
}

get_bbox <- function(xy) {
  lbottom <- c(min(xy[, 1]), min(xy[, 2]))
  rtop <- c(max(xy[, 1]), max(xy[, 2]))
  c(lbottom, rtop)
}

mv_to_null <- function(xy) {
  bbox <- get_bbox(xy)
  xy[, 1] <- xy[, 1] - bbox[1]
  xy[, 2] <- xy[, 2] - bbox[2]
  xy
}

scale_to_100 <- function(x) {
  a <- min(x)
  b <- max(x)
  100 / (b - a) * x - 100 / (b - a) * a
}

interpolate_cent <- function(cent, x) {
  a <- min(cent)
  b <- max(cent)
  alpha <- 100 / (b - a)
  beta <- -100 / (b - a) * a
  (x - beta) / alpha
}

map_to_angle_range <- function(xy, arange) {
  angles <- atan2(xy[, 2], xy[, 1]) / pi * 180
  angles[angles < 0] <- abs(angles[angles < 0]) + 180
  radii <- sqrt(rowSums(xy^2))
  angles <- normalise(angles, to = arange)
  angles <- angles * pi / 180
  cbind(radii * cos(angles), radii * sin(angles))
}

normalise <- function(x, from = range(x), to = c(0, 1)) {
  x <- (x - from[1]) / (from[2] - from[1])
  if (!identical(to, c(0, 1))) {
    x <- x * (to[2] - to[1]) + to[1]
  }
  x
}

get_seed <- function() {
  if (exists(".Random.seed", .GlobalEnv)) {
    return(.GlobalEnv$.Random.seed)
  } else {
    return(NULL)
  }
}

restore_seed <- function(oldseed) {
  if (!is.null(oldseed)) {
    .GlobalEnv$.Random.seed <- oldseed
  } else {
    rm(".Random.seed", envir = .GlobalEnv)
  }
}

iso_project <- function(xyz, a = 35.264, b = 45) {
  a <- a * pi / 180
  b <- b * pi / 180
  T1 <- matrix(
    c(1, 0, 0, 0, cos(a), sin(a), 0, -sin(a), cos(a)),
    3,
    3,
    byrow = TRUE
  )
  T2 <- matrix(
    c(cos(b), 0, -sin(b), 0, 1, 0, sin(b), 0, cos(b)),
    3,
    3,
    byrow = TRUE
  )
  trans <- T1 %*% T2 %*% t(xyz)
  coords2D <- matrix(c(1, 0, 0, 0, 1, 0, 0, 0, 0), 3, 3, byrow = TRUE) %*% trans
  coords2D <- t(coords2D)
  return(coords2D[, 1:2])
}

degree_lvl <- function(g) {
  if (!"lvl" %in% igraph::vertex_attr_names(g)) {
    stop(
      "level information should be stored in a vertex attribute called 'lvl'"
    )
  }
  A <- igraph::as_adj(g, "both", sparse = FALSE)
  lvl_mat <- outer(igraph::V(g)$lvl, igraph::V(g)$lvl, function(x, y) x == y)
  cent <- rbind(rowSums(A * lvl_mat), rowSums(A * !lvl_mat))
  rownames(cent) <- c("intra", "inter")
  cent
}

optim_isolates <- function(g, xyz) {
  deg <- degree_lvl(g)
  idx <- which(deg[1, ] == 0)
  if (length(idx) > 0) {
    neigh <- igraph::neighborhood(
      g,
      order = 1,
      nodes = idx,
      mode = "all",
      mindist = 1
    )
    xyz[idx, c(1, 3)] <- do.call(
      "rbind",
      lapply(neigh, function(id) cbind(mean(xyz[id, 1]), mean(xyz[id, 3])))
    )
  }
  xyz
}

optim_rotation <- function(g, xyz) {
  D <- igraph::distances(g)
  W <- 1 / D^2
  smin <- stress3D(xyz, W, D)
  # smin <- stress(xyz[,c(1,3)],W,D)
  amax <- 0
  idx <- which(igraph::V(g)$lvl == 2)
  for (alpha in seq(0, 360, 5)) {
    xyz_new <- xyz
    xyz_new[idx, c(1, 3)] <- layout_rotate(xyz_new[idx, c(1, 3)], alpha)
    stemp <- stress3D(xyz_new, W, D)
    # stemp <- stress(xyz_new[,c(1,3)],W,D)
    if (stemp < smin) {
      amax <- alpha
    }
  }
  xyz[idx, c(1, 3)] <- layout_rotate(xyz[idx, c(1, 3)], amax)
  xyz
}

optim_level <- function(g, lvl, xy) {
  A <- igraph::as_adj(g, "both")
  Ainter <- A[igraph::V(g)$lvl != lvl, igraph::V(g)$lvl == lvl]
  adjList <- apply(Ainter, 1, function(x) which(x == 1))

  xy2 <- do.call(
    "rbind",
    lapply(adjList, function(id) cbind(mean(xy[id, 1]), mean(xy[id, 2])))
  )
  idx <- is.na(xy2[, 1])

  mx <- mean(xy[, 1], na.rm = TRUE)
  my <- mean(xy[, 2], na.rm = TRUE)
  r <- max(sqrt((xy[, 1] - mx)^2 + (xy[, 2] - my)^2))

  if (length(idx) > 0) {
    xy2[idx, 1] <- stats::runif(n = sum(idx), min = mx - r, max = mx + r)
    xy2[idx, 2] <- sample(c(-1, 1), sum(idx), replace = T) *
      sqrt(r^2 - (xy2[idx, 1] - mx)^2) +
      my
  }
  xy2
}

find_lastD <- function(DList, i, j, k) {
  for (l in seq((k - 1), 1)) {
    if (l == 0) {
      next()
    }
    if (!is.infinite(DList[[l]][i, j])) {
      return(list(value = DList[[l]][i, j], index = l))
    }
  }
  return(list(value = Inf, index = NULL))
}

find_nextD <- function(DList, i, j, k) {
  for (l in seq((k + 1), length(DList))) {
    if (l > length(DList)) {
      break()
    }
    if (!is.infinite(DList[[l]][i, j])) {
      return(list(value = DList[[l]][i, j], index = l))
    }
  }
  return(list(value = Inf, index = NULL))
}

adjust_value <- function(lastD, nextD, k, tlast, tnext, n) {
  if (!is.infinite(lastD$value) && !is.infinite(nextD$value)) {
    beta <- (k - tlast) / (tnext - tlast)
    return((1 - beta) * lastD$value + beta * nextD$value + 1)
  } else if (is.infinite(lastD$value) && !is.infinite(nextD$value)) {
    return(nextD$value + 1)
  } else if (!is.infinite(lastD$value) && is.infinite(nextD$value)) {
    return(lastD$value + 1)
  } else {
    return(sqrt(n))
  }
}

adjust_dist <- function(DList) {
  n <- nrow(DList[[1]])
  for (k in seq_along(DList)) {
    for (i in 1:n) {
      for (j in 1:n) {
        if (is.infinite(DList[[k]][i, j])) {
          lastD <- find_lastD(DList, i, j, k)
          nextD <- find_nextD(DList, i, j, k)
          DList[[k]][i, j] <- adjust_value(
            lastD,
            nextD,
            k,
            lastD$index,
            nextD$index,
            n
          )
        }
      }
    }
  }
  DList
}

test_that("adjust_dist works", {
  D <- matrix(
    c(
      0,
      1,
      1,
      Inf,
      Inf,
      1,
      0,
      1,
      Inf,
      Inf,
      1,
      1,
      0,
      Inf,
      Inf,
      Inf,
      Inf,
      Inf,
      0,
      1,
      Inf,
      Inf,
      Inf,
      1,
      0
    ),
    5,
    5,
    byrow = TRUE
  )
  Dlist <- list(D, D)
  expect_no_error(adjust_dist(Dlist))
})

#' @title Metro Map Layout
#' @description Metro map layout based on multicriteria optimization
#' @param object original graph
#' @param xy initial layout of the original graph
#' @param l desired multiple of grid point spacing. (l*gr determines desired edge length)
#' @param gr grid spacing. (l*gr determines desired edge length)
#' @param w weight vector for criteria (see details)
#' @param bsize number of grid points a station can move away rom its original position
#' @details The function optimizes the following five criteria using a hill climbing algorithm:
#' - *Angular Resolution Criterion*: The angles of incident edges at each station should be maximized, because if there is only a small angle between any two adjacent edges, then it can become difficult to distinguish between them
#' - *Edge Length Criterion*: The edge lengths across the whole map should be approximately equal to ensure regular spacing between stations. It is based on the preferred multiple, l, of the grid spacing, g. The purpose of the criterion is to penalize edges that are longer than or shorter than lg.
#' - *Balanced Edge Length Criterion*: The length of edges incident to a particular station should be similar
#' - *Line Straightness Criterion*: (not yet implemented) Edges that form part of a line should, where possible, be co-linear either side of each station that the line passes through
#' - *Octilinearity Criterion*: Each edge should be drawn horizontally, vertically, or diagonally at 45 degree, so we penalize edges that are not at a desired angle
#' @return new coordinates for stations
#' @references
#' Stott, Jonathan, et al. "Automatic metro map layout using multicriteria optimization." IEEE Transactions on Visualization and Computer Graphics 17.1 (2010): 101-114.
#' @author David Schoch
#' @examples
#' # the algorithm has problems with parallel edges
#' library(igraph)
#' g <- simplify(metro_berlin)
#' xy <- cbind(V(g)$lon, V(g)$lat) * 100
#'
#' # the algorithm is not very stable. try playing with the parameters
#' \dontrun{
#' xy_new <- layout_as_metromap(g, xy, l = 2, gr = 0.5, w = c(100, 100, 1, 1, 100), bsize = 35)
#' }
#' @export
layout_as_metromap <- function(object, xy, l = 2, gr = 0.0025, w = rep(1, 5), bsize = 5) {
    adj <- as_adj_list1(object)
    adj <- lapply(adj, function(x) x - 1)
    adj_deg2 <- adj[unlist(lapply(adj, length)) == 2]
    el <- igraph::get.edgelist(object, FALSE) - 1

    xy <- snap_to_grid(xy, gr)

    bbox <- station_bbox(xy, bsize, gr)

    xy_new <- layout_as_metro_iter(adj, el, adj_deg2, xy, bbox, l, gr, w, bsize)
    xy_new
}

# helper ----
snap_to_grid <- function(xy, gr) {
    xmin <- min(xy[, 1])
    xmax <- max(xy[, 1])
    ymin <- min(xy[, 2])
    ymax <- max(xy[, 2])

    deltax <- seq(xmin - 4 * gr, xmax + 4 * gr, by = gr)
    deltay <- seq(ymin - 4 * gr, ymax + 4 * gr, by = gr)

    xdiff <- outer(xy[, 1], deltax, function(x, y) abs(x - y))
    ydiff <- outer(xy[, 2], deltay, function(x, y) abs(x - y))

    xy_new <- cbind(deltax[apply(xdiff, 1, which.min)], deltay[apply(ydiff, 1, which.min)])
    dups <- duplicated(xy_new)
    while (any(dups)) {
        xy_new[which(dups), ] <- xy_new[which(dups), ] + c(sample(c(1, -1), 1) * gr, sample(c(1, -1), 1) * gr)
        dups <- duplicated(xy_new)
    }
    xy_new
}

station_bbox <- function(xy, bsize, gr) {
    cbind(xy - bsize * gr, xy + bsize * gr)
}

as_adj_list1 <- function(g) {
    n <- igraph::vcount(g)
    lapply(1:n, function(i) {
        x <- g[[i]][[1]]
        attr(x, "env") <- NULL
        attr(x, "graph") <- NULL
        class(x) <- NULL
        x
    })
}

#' graphlayouts: layout algorithms for network visualizations
#'
#' @description The package implements several new layout algorithms to visualize networks.
#' Most are based on the concept of stress majorization. Some more specific algorithms allow to emphasize
#' hidden group structures in networks or focus on specific nodes. The package is best used in conjunction with
#' ggraph.
#'
#' Some features of the package are:
#'
#' \itemize{
#' \item `layout_with_stress()` is a state of the art deterministic layout algorithms.
#' \item `layout_as_backbone()` uncovers hidden group structures (if they exist) by emphasizing strongly embedded edges.
#' \item `layout_with_focus()` and `layout_with_centrality()` produce concentric layouts with a focal or most central nodes in the center.
#' \item `layout_with_eigen()` implements some layout algorithms on the basis of eigenvectors
#' \item `layout_with_sparse_stress()` sparse stress for large graphs
#' \item `layout_with_pmds()` pivot MDS for large graphs.
#' \item `layout_as_dynamic()` for longitudinal network data
#' }
#'
#' A detailed tutorial can be found at <https://schochastics.github.io/netVizR/>
#' @keywords internal
"_PACKAGE"

# resolve edge weights into branch lengths, following the same NA/NULL/vector
# semantics as the stress layouts (see .layout_with_stress_dim)
.resolve_branch_lengths <- function(g, weights) {
  m <- igraph::ecount(g)
  if (is.null(weights)) {
    if ("weight" %in% igraph::edge_attr_names(g)) {
      return(as.numeric(igraph::E(g)$weight))
    }
    return(rep(1, m))
  }
  if (length(weights) == 1 && is.na(weights)) {
    return(rep(1, m))
  }
  if (length(weights) != m) {
    stop("weights must be a numeric vector of length ecount(g)", call. = FALSE)
  }
  as.numeric(weights)
}

# compute coordinates for a single tree (connected, acyclic subgraph)
.tree_unrooted_component <- function(
  sg,
  w_sub,
  mode,
  daylight_iter,
  iter,
  tol
) {
  n <- igraph::vcount(sg)
  if (n == 1) {
    return(matrix(c(0, 0), 1, 2))
  }
  if (n == 2) {
    return(matrix(c(0, 0, w_sub[1], 0), 2, 2, byrow = TRUE))
  }

  if (mode == "stress") {
    # stress majorization on patristic (tree path-length) distances
    D <- igraph::distances(sg, weights = w_sub)
    W <- 1 / D^2
    diag(W) <- 0
    xinit <- .init_layout(sg, D, mds = TRUE, n = n, dim = 2)
    return(stress_major(xinit, W, D, iter, tol))
  }

  # equal-angle / equal-daylight: build a rooted traversal
  root <- which.max(igraph::degree(sg))[1]
  res <- igraph::bfs(sg, root = root, parent = TRUE, order = TRUE)
  ord <- as.integer(res$order)
  father <- as.integer(res$parent)

  # subtree leaf counts (accumulate over reverse BFS order)
  is_parent <- logical(n)
  is_parent[father[!is.na(father)]] <- TRUE
  leaf_count <- integer(n)
  leaf_count[!is_parent] <- 1L
  for (v in rev(ord)) {
    f <- father[v]
    if (!is.na(f)) {
      leaf_count[f] <- leaf_count[f] + leaf_count[v]
    }
  }
  nleaves <- leaf_count[root]

  # branch length from each node to its parent (root: 0)
  branch_len <- numeric(n)
  non_root <- which(!is.na(father))
  if (length(non_root) > 0) {
    eids <- igraph::get_edge_ids(
      sg,
      as.vector(rbind(father[non_root], non_root))
    )
    branch_len[non_root] <- w_sub[eids]
  }

  parent0 <- father - 1L
  parent0[is.na(parent0)] <- -1L

  xy <- equal_angle_layout(
    ord - 1L,
    parent0,
    leaf_count,
    branch_len,
    nleaves
  )

  if (mode == "equaldaylight") {
    al <- igraph::as_adj_list(sg, mode = "all")
    adj_ptr <- integer(n + 1)
    for (i in seq_len(n)) {
      adj_ptr[i + 1] <- adj_ptr[i] + length(al[[i]])
    }
    adj_idx <- as.integer(unlist(al)) - 1L
    xy <- equal_daylight_layout(adj_ptr, adj_idx, ord - 1L, xy, daylight_iter)
  }

  xy
}

#' unrooted tree layout
#'
#' @name layout_tree_unrooted
#' @description arranges the nodes of a tree (or forest) in the plane without a
#' designated root, using algorithms developed for drawing phylogenies.
#' @param g igraph object. Each connected component must be a tree (acyclic).
#' @param mode which algorithm to use. One of `"equalangle"` (default),
#' `"equaldaylight"` or `"stress"`. See details.
#' @param weights possibly a numeric vector with edge weights, interpreted as
#' branch lengths. If this is NULL and the graph has a weight edge attribute,
#' then the attribute is used. If this is NA then unit branch lengths are used
#' (even if the graph has a weight attribute). By default, unit branch lengths
#' are used.
#' @param daylight_iter number of refinement sweeps for `mode = "equaldaylight"`.
#' @param iter number of iterations during stress optimization (only used for
#' `mode = "stress"`).
#' @param tol stopping criterion for stress optimization (only used for
#' `mode = "stress"`).
#' @param bbox width of layout. Only relevant to determine the placement of
#' disconnected components (forests).
#' @details Three algorithms are available:
#'
#' * `"equalangle"` (Felsenstein, 1989): each subtree is allotted an angular
#'   wedge proportional to its number of leaves. Linear time, but can look
#'   crowded on unbalanced trees.
#' * `"equaldaylight"` (Felsenstein): iteratively equalizes the angular gaps
#'   ("daylight") around each internal node, starting from the equal-angle
#'   layout, for a more balanced drawing.
#' * `"stress"`: stress majorization using the patristic (tree path-length)
#'   distances as target distances, reusing the package's stress engine.
#'
#' Branch lengths (edge weights) are respected when present; otherwise unit
#' lengths are used.
#'
#' The layout_igraph_* function should not be used directly. It is only used as
#' an argument for plotting with 'igraph'. 'ggraph' natively supports the layout.
#' @return matrix of xy coordinates
#' @references Felsenstein, J. (1989). PHYLIP - Phylogeny Inference Package
#' (Version 3.2). *Cladistics*, 5, 164-166.
#'
#' Bachmaier, C., Brandes, U., & Schlieper, B. (2005). Drawing phylogenetic
#' trees. *In International Symposium on Algorithms and Computation* (pp.
#' 1110-1121). Springer, Berlin, Heidelberg.
#' @examples
#' library(igraph)
#' g <- make_tree(20, 3, mode = "undirected")
#'
#' xy <- layout_as_tree_unrooted(g)
#' xy <- layout_as_tree_unrooted(g, mode = "equaldaylight")
#' @export
layout_as_tree_unrooted <- function(
  g,
  mode = c("equalangle", "equaldaylight", "stress"),
  weights = NA,
  daylight_iter = 15,
  iter = 500,
  tol = 0.0001,
  bbox = 30
) {
  ensure_igraph(g)
  mode <- match.arg(mode)

  oldseed <- get_seed()
  set.seed(42)
  on.exit(restore_seed(oldseed))

  w_full <- .resolve_branch_lengths(g, weights)

  comps <- igraph::components(g, "weak")

  # tag edges so weights can be mapped onto induced subgraphs
  igraph::edge_attr(g, "_edgename") <- seq_len(igraph::ecount(g))
  names(w_full) <- seq_len(igraph::ecount(g))

  lg <- list()
  node_order <- integer(0)

  for (i in seq_len(comps$no)) {
    idx <- which(comps$membership == i)
    sg <- igraph::induced_subgraph(g, idx)
    node_order <- c(node_order, idx)

    if (igraph::ecount(sg) != igraph::vcount(sg) - 1) {
      stop(
        "each connected component of g must be a tree (acyclic)",
        call. = FALSE
      )
    }

    edge_idx <- igraph::edge_attr(g, "_edgename") %in%
      igraph::edge_attr(sg, "_edgename")
    w_sub <- unname(w_full[edge_idx])

    lg[[i]] <- .tree_unrooted_component(
      sg,
      w_sub,
      mode,
      daylight_iter,
      iter,
      tol
    )
  }

  if (comps$no == 1) {
    return(lg[[1]])
  }

  lg <- lapply(lg, mv_to_null)
  p <- order(comps$csize)
  lg <- .component_mover(lg, p, bbox)
  x <- do.call("rbind", lg)
  x[order(node_order), , drop = FALSE]
}

test_that("equalangle works on a balanced tree", {
  g <- igraph::make_tree(15, 2, mode = "undirected")
  expect_silent(
    r <- layout_as_tree_unrooted(g)
  )
  expect_is(r, "matrix")
  expect_equal(dim(r), c(15, 2))
  expect_true(all(is.finite(r)))
})

test_that("all modes work and return n x 2 finite matrices", {
  g <- igraph::make_tree(20, 3, mode = "undirected")
  for (m in c("equalangle", "equaldaylight", "stress")) {
    expect_silent(
      r <- layout_as_tree_unrooted(g, mode = m)
    )
    expect_is(r, "matrix")
    expect_equal(dim(r), c(20, 2))
    expect_true(all(is.finite(r)))
  }
})

test_that("it works on a star", {
  g <- igraph::make_star(8, mode = "undirected", center = 1)
  expect_silent(
    r <- layout_as_tree_unrooted(g, mode = "equaldaylight")
  )
  expect_is(r, "matrix")
  expect_equal(dim(r), c(8, 2))
})

test_that("it works on an unbalanced caterpillar tree", {
  g <- igraph::make_graph(~ 1 - 2, 2 - 3, 3 - 4, 4 - 5, 2 - 6, 3 - 7, 4 - 8)
  expect_silent(
    r <- layout_as_tree_unrooted(g, mode = "equaldaylight")
  )
  expect_is(r, "matrix")
  expect_equal(dim(r), c(igraph::vcount(g), 2))
  expect_true(all(is.finite(r)))
})

test_that("branch lengths are respected", {
  g <- igraph::make_star(5, mode = "undirected", center = 1)
  igraph::E(g)$weight <- c(1, 2, 3, 4)
  r <- layout_as_tree_unrooted(g, weights = NULL)
  # distance from center (node 1) to each leaf should match the branch length
  d <- sqrt(rowSums((r[-1, ] - matrix(r[1, ], 4, 2, byrow = TRUE))^2))
  expect_equal(sort(d), c(1, 2, 3, 4))
})

test_that("weights = NA ignores the weight attribute", {
  g <- igraph::make_tree(10, 2, mode = "undirected")
  igraph::E(g)$weight <- runif(igraph::ecount(g), 1, 5)
  g2 <- igraph::delete_edge_attr(g, "weight")
  expect_equal(
    layout_as_tree_unrooted(g, weights = NA),
    layout_as_tree_unrooted(g2, weights = NA)
  )
  expect_false(isTRUE(all.equal(
    layout_as_tree_unrooted(g, weights = NA),
    layout_as_tree_unrooted(g, weights = NULL)
  )))
})

test_that("it works on a forest", {
  g <- igraph::disjoint_union(
    igraph::make_tree(7, 2, mode = "undirected"),
    igraph::make_star(5, mode = "undirected")
  )
  expect_silent(
    r <- layout_as_tree_unrooted(g)
  )
  expect_is(r, "matrix")
  expect_equal(dim(r), c(12, 2))
  expect_true(all(is.finite(r)))
})

test_that("small trees work", {
  expect_equal(dim(layout_as_tree_unrooted(igraph::make_graph(~a))), c(1, 2))
  expect_equal(dim(layout_as_tree_unrooted(igraph::make_graph(~ a - -b))), c(2, 2))
})

test_that("errors on invalid input", {
  expect_error(layout_as_tree_unrooted(5))
  # a graph with a cycle is not a tree
  expect_error(layout_as_tree_unrooted(igraph::make_ring(5)))
  expect_error(layout_as_tree_unrooted(igraph::make_full_graph(4)))
})

test_that("igraph wrapper returns a data frame with coordinates", {
  g <- igraph::make_tree(10, 2, mode = "undirected")
  igraph::V(g)$name <- letters[1:10]
  nodes <- layout_igraph_tree_unrooted(g)
  expect_s3_class(nodes, "data.frame")
  expect_true(all(c("x", "y", "circular") %in% names(nodes)))
  expect_equal(nrow(nodes), 10)
})

#' dynamic graph layout
#' @description Create layouts for longitudinal networks.
#' @name layout_dynamic
#' @param gList list of igraph objects. Each network must contain the same set of nodes.
#' @param weights possibly a numeric vector with edge weights. If this is NULL and the graph has a weight edge attribute, then the attribute is used. If this is NA then no weights are used (even if the graph has a weight attribute). By default, weights are ignored. See details for more.
#' @param alpha weighting of reference layout. See details.
#' @param iter number of iterations during stress optimization
#' @param tol stopping criterion for stress optimization
#' @details The reference layout is calculated based on the union of all graphs. The parameter alpha controls the influence of the reference layout.
#' For alpha=1, only the reference layout is used and all graphs have the same layout. For alpha=0, the stress layout of each individual graph is used. Values in-between interpolate between the two layouts.
#'
#' Be careful when using weights. In most cases, the inverse of the edge weights should be used to ensure that the endpoints of an edges with higher weights are closer together (weights=1/E(g)$weight).
#' @return list of coordinates for each graph
#' @references Brandes, U. and Indlekofer, N. and Mader, M. (2012). Visualization methods for longitudinal social networks and stochastic actor-oriented modeling. *Social Networks* 34 (3) 291-308
#' @examples
#' library(igraph)
#' g1 <- sample_gnp(20, 0.2)
#' g2 <- sample_gnp(20, 0.2)
#' g3 <- sample_gnp(20, 0.2)
#'
#' xy <- layout_as_dynamic(list(g1, g2, g3))
#'
#' # layout for first network
#' xy[[1]]
#' @export
layout_as_dynamic <- function(gList, weights = NA, alpha = 0.5, iter = 500, tol = 1e-04) {
    if (igraph::is_igraph(gList)) {
        stop("'gList' must be a list of igraph objects.")
    }
    check_networks <- vapply(gList, FUN = function(x) igraph::is_igraph(x), FUN.VALUE = FALSE)
    if (!all(check_networks)) {
        stop("'gList' must be a list of igraph objects.")
    }

    # prepare reference layout
    g <- Reduce(igraph::union, gList)
    check_nodes <- vapply(gList, FUN = function(x) igraph::vcount(x) == igraph::vcount(g), FUN.VALUE = FALSE)
    if (!all(check_nodes)) {
        stop("all nodes must be present in each network")
    }
    n <- igraph::vcount(g)
    DList <- lapply(gList, igraph::distances, weights = weights)
    DList <- adjust_dist(DList)
    Dmean <- Reduce("+", DList) / length(DList)
    Dvar <- Reduce("+", lapply(DList, function(x) (x - Dmean)^2)) / length(DList)
    W <- 1 / Dmean^2 + 1 / (1 + Dvar)
    diag(W) <- 0

    # calculate reference layout
    rmat <- matrix(stats::runif(n * 2, -0.1, 0.1), n, 2)
    xinit <- igraph::layout_with_mds(g) + rmat
    xref <- stress_major(xinit, W, Dmean, iter, tol)

    xycoords <- vector("list", length(gList))
    for (i in seq_along(gList)) {
        D <- DList[[i]]
        W <- 1 / D^2
        diag(W) <- 0
        if (i == 1) {
            xycoords[[i]] <- stress_major(xref, W, D, iter, tol)
        } else {
            xycoords[[i]] <- stress_major(xycoords[[i - 1]], W, D, iter, tol)
        }
        xycoords[[i]] <- (1 - alpha) * xycoords[[i]] + alpha * xref
    }
    xycoords
}

test_that("layout_dynamic works", {
  library(igraph)
  expect_error(layout_as_dynamic(3))
  expect_error(layout_as_dynamic(list(
    igraph::graph.full(3),
    igraph::graph.full(5)
  )))
  expect_is(
    layout_as_dynamic(list(igraph::graph.full(5), igraph::graph.full(5))),
    "list"
  )
  expect_no_error(layout_as_dynamic(list(igraph::graph.full(3))))
  expect_error(layout_as_dynamic(igraph::graph.full(3)))
})

#' UMAP graph layouts
#' @description Using the UMAP dimensionality reduction algorithm as a graph layout
#' @name layout_umap
#' @param g igraph object
#' @param pivots if not NULL, number of pivot nodes to use for distance calculation (for large graphs).
#' @param ... additional parameters for umap. See the `?uwot::umap` for help.
#' @details The layout_igraph_* function should not be used directly. It is only used as an argument for plotting with 'igraph'. UMAP can be tuned by many different parameters. Refer to the documentation at https://github.com/jlmelville/uwot for help
#' @references
#' McInnes, Leland, John Healy, and James Melville. "Umap: Uniform manifold approximation and projection for dimension reduction." arXiv preprint arXiv:1802.03426 (2018).
#' @author David Schoch
#' @return matrix of xy coordinates
#' @examples
#' library(igraph)
#'
#' g <- sample_islands(10, 20, 0.6, 10)
#' # xy <- layout_with_umap(g, min_dist = 0.5)
#' @export

layout_with_umap <- function(g, pivots = NULL, ...) {
    if (!requireNamespace("uwot", quietly = TRUE)) {
        stop("uwot is needed for this function to work. Please install it.", call. = FALSE)
    }
    ensure_igraph(g)
    if (is.null(pivots)) {
        D <- igraph::distances(g)
    } else {
        pivs <- sample(1:igraph::vcount(g), pivots)
        D <- t(igraph::distances(g, v = pivs))
    }
    uwot::umap(D, ...)
}

.init_layout <- function(g, D, mds, n, dim) {
    if (!mds) {
        return(matrix(stats::runif(n * dim, 0, 1), n, dim))
    } else {
        rmat <- matrix(stats::runif(n * dim, -0.1, 0.1), n, dim)
        if (n <= 100) {
            return(igraph::layout_with_mds(g, dim = dim) + rmat)
        } else {
            return(layout_with_pmds(g, D = D[, sample(1:n, 100)], dim = dim) + rmat)
        }
    }
}

.component_mover <- function(lg, p, bbox) {
    curx <- 0
    cury <- 0
    maxy <- 0
    for (comp in p) {
        if (curx + max(lg[[comp]][, 1]) > bbox) {
            curx <- 0
            cury <- maxy + 1
        }
        lg[[comp]][, 1] <- lg[[comp]][, 1] + curx
        lg[[comp]][, 2] <- lg[[comp]][, 2] + cury
        curx <- max(lg[[comp]][, 1]) + 1
        maxy <- max(c(maxy, max(lg[[comp]][, 2])))
    }
    return(lg)
}

.component_layouter <- function(g, weights, comps, dim, mds, bbox, iter, tol, FUN, ...) {
    # check which ... are arguments of FUN
    FUN <- match.fun(FUN)
    params_in <- list(...)
    FUN_formals <- formals(FUN)
    idx <- names(params_in) %in% names(FUN_formals)
    params <- params_in[idx]
    if ("dim" %in% names(FUN_formals)) {
        params <- c(params, list(dim = params_in[["fixdim"]]))
    }

    lg <- list()
    node_order <- c()
    if (!is.null(weights) && any(!is.na(weights))) {
        igraph::edge_attr(g, "_edgename") <- 1:igraph::ecount(g)
        names(weights) <- 1:igraph::ecount(g)
    }

    for (i in 1:comps$no) {
        idx <- which(comps$membership == i)
        sg <- igraph::induced_subgraph(g, idx)
        edge_idx <- igraph::edge_attr(g, "_edgename") %in% igraph::edge_attr(sg, "_edgename")
        n <- igraph::vcount(sg)
        node_order <- c(node_order, idx)

        if (n == 1) {
            lg[[i]] <- matrix(rep(0, dim), 1, dim, byrow = TRUE)
        } else if (n == 2) {
            lg[[i]] <- matrix(c(0, rep(0, dim - 1), 1, rep(0, dim - 1)), 2, dim, byrow = TRUE)
            next()
        } else {
            if (!is.null(weights) && any(!is.na(weights))) {
                D <- igraph::distances(sg, weights = weights[edge_idx])
            } else {
                D <- igraph::distances(sg, weights = weights)
            }
            W <- 1 / D^2
            diag(W) <- 0

            xinit <- .init_layout(sg, D, mds, n, dim)
            if ("dim" %in% names(params)) {
                xinit[, params[["dim"]]] <- params_in[["coord"]][idx]
            }
            params_FUN <- c(params, list(y = xinit, W = W, D = D, iter = iter, tol = tol))
            lg[[i]] <- do.call(FUN, params_FUN) # FUN(xinit, W, D, iter, tol)
        }
    }
    if (!"dim" %in% names(params)) {
        lg <- lapply(lg, mv_to_null)
        p <- order(comps$csize)
        lg <- .component_mover(lg, p, bbox)
    }
    x <- do.call("rbind", lg)
    x[order(node_order), , drop = FALSE]
}

.layout_with_stress_dim <- function(g, weights = NA, iter = 500, tol = 0.0001, mds = TRUE, bbox = 30, dim = 2) {
    ensure_igraph(g)
    if (!dim %in% c(2, 3)) {
        stop("dim must be either 2 or 3")
    }

    oldseed <- get_seed()
    set.seed(42) # stress is deterministic and produces the same result up to translation. This keeps the layout fixed
    on.exit(restore_seed(oldseed))

    comps <- igraph::components(g, "weak")
    if (comps$no == 1) {
        n <- igraph::vcount(g)

        if (n == 1) {
            return(matrix(rep(0, dim), 1, dim, byrow = TRUE))
        } else if (n == 2) {
            return(matrix(c(0, rep(0, dim - 1), 1, rep(0, dim - 2)), 2, dim, byrow = TRUE))
        } else {
            if (!is.null(weights) && any(!is.na(weights))) {
                D <- igraph::distances(g, weights = weights)
            } else {
                D <- igraph::distances(g)
            }
            W <- 1 / D^2
            diag(W) <- 0

            xinit <- .init_layout(g, D, mds, n, dim)

            if (dim == 2) {
                return(stress_major(xinit, W, D, iter, tol))
            } else {
                return(stress_major3D(xinit, W, D, iter, tol))
            }
        }
    } else {
        layouter <- ifelse(dim == 2, stress_major, stress_major3D)
        return(.component_layouter(
            g = g, weights = weights, comps = comps, dim = dim, mds = mds,
            bbox = bbox, iter = iter, tol = tol, FUN = layouter
        ))
    }
}


#' stress majorization layout
#'
#' @name layout_stress
#' @rdname layout_stress
#' @description force-directed graph layout based on stress majorization.
#' Similar to Kamada-Kawai, but generally faster and with better results.
#' @param g igraph object
#' @param weights possibly a numeric vector with edge weights. If this is NULL and the graph has a weight edge attribute, then the attribute is used. If this is NA then no weights are used (even if the graph has a weight attribute). By default, weights are ignored. See details for more.
#' @param iter number of iterations during stress optimization
#' @param tol stopping criterion for stress optimization
#' @param mds should an MDS layout be used as initial layout (default: TRUE)
#' @param bbox width of layout. Only relevant to determine the placement of disconnected graphs
#' @details Be careful when using weights. In most cases, the inverse of the edge weights should be used to ensure that the endpoints of an edges with higher weights are closer together (weights=1/E(g)$weight).
#'
#' The layout_igraph_* function should not be used directly. It is only used as an argument for plotting with 'igraph'.
#' 'ggraph' natively supports the layout.
#' @seealso [layout_stress3D]
#' @return matrix of xy coordinates
#' @references Gansner, E. R., Koren, Y., & North, S. (2004). Graph drawing by stress majorization. *In International Symposium on Graph Drawing* (pp. 239-250). Springer, Berlin, Heidelberg.
#' @examples
#' library(igraph)
#' set.seed(665)
#'
#' g <- sample_pa(100, 1, 1, directed = FALSE)
#'
#' # calculate layout manually
#' xy <- layout_with_stress(g)
#'
#' # use it with ggraph
#' \dontrun{
#' library(ggraph)
#' ggraph(g, layout = "stress") +
#'     geom_edge_link0(edge_width = 0.2, colour = "grey") +
#'     geom_node_point(col = "black", size = 0.3) +
#'     theme_graph()
#' }
#' @export
layout_with_stress <- function(g, weights = NA, iter = 500, tol = 0.0001, mds = TRUE, bbox = 30) {
    .layout_with_stress_dim(g, weights, iter, tol, mds, bbox, dim = 2)
}

#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#

#' stress majorization layout in 3D
#'
#' @name layout_stress3D
#' @rdname layout_stress3D
#' @description force-directed graph layout based on stress majorization in 3D.
#' @param g igraph object
#' @param weights possibly a numeric vector with edge weights. If this is NULL and the graph has a weight edge attribute, then the attribute is used. If this is NA then no weights are used (even if the graph has a weight attribute). By default, weights are ignored. See details for more.
#' @param iter number of iterations during stress optimization
#' @param tol stopping criterion for stress optimization
#' @param mds should an MDS layout be used as initial layout (default: TRUE)
#' @param bbox width of layout. Only relevant to determine the placement of disconnected graphs
#' @details Be careful when using weights. In most cases, the inverse of the edge weights should be used to ensure that the endpoints of an edges with higher weights are closer together (weights=1/E(g)$weight).
#'
#' @return matrix of xyz coordinates
#' @seealso [layout_stress]
#' @references Gansner, E. R., Koren, Y., & North, S. (2004). Graph drawing by stress majorization. *In International Symposium on Graph Drawing* (pp. 239-250). Springer, Berlin, Heidelberg.
#' @export
layout_with_stress3D <- function(g, weights = NA, iter = 500, tol = 0.0001, mds = TRUE, bbox = 30) {
    .layout_with_stress_dim(g, weights, iter, tol, mds, bbox, dim = 3)
}


#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#' radial focus layout
#'
#' @description arrange nodes in concentric circles around a focal node according to their distance from the focus.
#'
#' @name layout_focus
#' @param g igraph object
#' @param v id of focal node to be placed in the center
#' @param weights possibly a numeric vector with edge weights. If this is NULL and the graph has a weight edge attribute, then the attribute is used. If this is NA then no weights are used (even if the graph has a weight attribute). By default, weights are ignored. See details for more.
#' @param iter number of iterations during stress optimization
#' @param tol stopping criterion for stress optimization
#' @details Be careful when using weights. In most cases, the inverse of the edge weights should be used to ensure that the endpoints of an edges with higher weights are closer together (weights=1/E(g)$weight).
#' @seealso [layout_focus_group]
#' The layout_igraph_* function should not be used directly. It is only used as an argument for plotting with 'igraph'.
#' 'ggraph' natively supports the layout.
#' @return a list containing xy coordinates and the distances to the focal node
#' @references Brandes, U., & Pich, C. (2011). More flexible radial layout. *Journal of Graph Algorithms and Applications*, 15(1), 157-173.
#' @examples
#' library(igraph)
#' g <- sample_gnp(10, 0.4)
#' coords <- layout_with_focus(g, v = 1)
#' coords
#' @export

layout_with_focus <- function(g, v, weights = NA, iter = 500, tol = 0.0001) {
    ensure_igraph(g)
    ensure_connected(g)
    if (missing(v)) {
        stop("v missing without a default")
    }
    oldseed <- get_seed()
    set.seed(42) # stress is deterministic and produces same result up to translation. This keeps the layout fixed
    on.exit(restore_seed(oldseed))

    n <- igraph::vcount(g)
    D <- igraph::distances(g, weights = weights)
    W <- 1 / D^2
    diag(W) <- 0

    Z <- matrix(0, n, n)
    Z[v, ] <- Z[, v] <- 1
    Z <- W * Z


    rmat <- matrix(stats::runif(n * 2, -0.1, 0.1), n, 2)
    xinit <- igraph::layout_with_mds(g) + rmat

    tseq <- seq(0, 1, 0.1)
    x <- stress_focus(xinit, W, D, Z, tseq, iter, tol)

    offset <- x[v, ]
    x <- t(apply(x, 1, function(x) x - offset))
    return(list(xy = x, distance = D[, v]))
}

#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#' radial centrality layout
#'
#' @description arranges nodes in concentric circles according to a centrality index.
#'
#' @name layout_centrality
#' @param g igraph object
#' @param cent centrality scores
#' @param scale logical. should centrality scores be scaled to \eqn{[0,100]}? (Default: TRUE)
#' @param iter number of iterations during stress optimization
#' @param tol stopping criterion for stress optimization
#' @param tseq numeric vector. increasing sequence of coefficients to combine regular stress and constraint stress. See details.
#' @details The function optimizes a convex combination of regular stress and a constrained stress function which forces
#' nodes to be arranged on concentric circles. The vector `tseq` is the sequence of parameters used for the convex combination.
#' In iteration i of the algorithm \eqn{tseq[i]} is used to combine regular and constraint stress as \eqn{(1-tseq[i])*stress_{regular}+tseq[i]*stress_{constraint}}. The sequence must be increasing, start at zero and end at one. The default setting should be a good choice for most graphs.
#'
#' The layout_igraph_* function should not be used directly. It is only used as an argument for plotting with 'igraph'.
#' 'ggraph' natively supports the layout.
#' @seealso [layout_centrality_group]
#' @return matrix of xy coordinates
#' @references Brandes, U., & Pich, C. (2011). More flexible radial layout. Journal of Graph Algorithms and Applications, 15(1), 157-173.
#' @examples
#' library(igraph)
#'
#' g <- sample_gnp(10, 0.4)
#' \dontrun{
#' library(ggraph)
#' ggraph(g, layout = "centrality", centrality = closeness(g)) +
#'     draw_circle(use = "cent") +
#'     geom_edge_link0() +
#'     geom_node_point(shape = 21, fill = "grey25", size = 5) +
#'     theme_graph() +
#'     coord_fixed()
#' }

#' @export
#'
layout_with_centrality <- function(g, cent, scale = TRUE, iter = 500, tol = 0.0001, tseq = seq(0, 1, 0.2)) {
    ensure_igraph(g)
    ensure_connected(g)
    if (missing(cent)) {
        stop("cent missing without a default")
    }
    oldseed <- get_seed()
    set.seed(42) # stress is deterministic and produces same result up to translation. This keeps the layout fixed
    on.exit(restore_seed(oldseed))

    n <- igraph::vcount(g)
    if (scale) {
        cent <- scale_to_100(cent)
    }
    r <- unname(igraph::diameter(g) / 2 * (1 - ((cent - min(cent)) / (max(cent) - min(cent) + 1))))

    D <- igraph::distances(g, weights = NA)
    W <- 1 / D^2
    diag(W) <- 0

    rmat <- matrix(stats::runif(n * 2, -0.1, 0.1), n, 2)
    xinit <- igraph::layout_with_mds(g) + rmat

    x <- stress_major(xinit, W, D, iter, tol)
    x <- stress_radii(x, W, D, r, tseq)

    # move highest cent to 0,0
    idx <- which.max(cent)[1]
    offset <- x[idx, ]

    x <- t(apply(x, 1, function(x) x - offset))
    if (scale) {
        radii_new <- round(100 - cent, 8)
        angles <- apply(x, 1, function(y) atan2(y[2], y[1]))
        return(cbind(radii_new * cos(angles), radii_new * sin(angles)))
    } else {
        radii_new <- round(max(cent) - cent, 8)
        angles <- apply(x, 1, function(y) atan2(y[2], y[1]))
        return(cbind(radii_new * cos(angles), radii_new * sin(angles)))
    }
}

#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#' constrained stress layout
#'
#' @name layout_constrained_stress
#' @description force-directed graph layout based on stress majorization with variable constrained
#' @param g igraph object
#' @param coord numeric vector. fixed coordinates for dimension specified in `fixdim`.
#' @param fixdim string. which dimension should be fixed. Either "x" or "y".
#' @param weights possibly a numeric vector with edge weights. If this is NULL and the graph has a weight edge attribute, then the attribute is used. If this is NA then no weights are used (even if the graph has a weight attribute). By default, weights are ignored. See details for more.
#' @param iter number of iterations during stress optimization
#' @param tol stopping criterion for stress optimization
#' @param mds should an MDS layout be used as initial layout (default: TRUE)
#' @param bbox constrain dimension of output. Only relevant to determine the placement of disconnected graphs
#' @details Be careful when using weights. In most cases, the inverse of the edge weights should be used to ensure that the endpoints of an edges with higher weights are closer together (weights=1/E(g)$weight).
#'
#' The layout_igraph_* function should not be used directly. It is only used as an argument for plotting with 'igraph'.
#' 'ggraph' natively supports the layout.
#' @seealso [layout_constrained_stress3D]
#' @return matrix of xy coordinates
#' @references Gansner, E. R., Koren, Y., & North, S. (2004). Graph drawing by stress majorization. *In International Symposium on Graph Drawing* (pp. 239-250). Springer, Berlin, Heidelberg.
#' @export
layout_with_constrained_stress <- function(g, coord, fixdim = "x", weights = NA,
                                           iter = 500, tol = 0.0001, mds = TRUE, bbox = 30) {
    ensure_igraph(g)

    oldseed <- get_seed()
    set.seed(42) # stress is deterministic and produces same result up to translation. This keeps the layout fixed
    on.exit(restore_seed(oldseed))

    fixdim <- match.arg(fixdim, c("x", "y"))
    fixdim <- ifelse(fixdim == "x", 1, 2)

    if (missing(coord)) {
        stop('"coord" is missing with no default.')
    }
    comps <- igraph::components(g, "weak")
    if (comps$no == 1) {
        if (igraph::vcount(g) == 1) {
            return(matrix(c(0, 0), 1, 2))
        } else {
            D <- igraph::distances(g, weights = weights)
            W <- 1 / D^2
            diag(W) <- 0
            n <- igraph::vcount(g)
            xinit <- .init_layout(g, D, mds, n, dim = 2)
            xinit[, fixdim] <- coord
            return(constrained_stress_major(xinit, fixdim, W, D, iter, tol))
        }
    } else {
        return(.component_layouter(
            g = g, weights = weights, comps = comps, dim = 2, mds = mds,
            bbox = bbox, iter = iter, tol = tol, FUN = constrained_stress_major, fixdim = fixdim, coord = coord
        ))
    }
}

#' constrained stress layout in 3D
#'
#' @name layout_constrained_stress3D
#' @description force-directed graph layout based on stress majorization with variable constrained in 3D
#' @param g igraph object
#' @param coord numeric vector. fixed coordinates for dimension specified in `fixdim`.
#' @param fixdim string. which dimension should be fixed. Either "x", "y" or "z".
#' @param weights possibly a numeric vector with edge weights. If this is NULL and the graph has a weight edge attribute, then the attribute is used. If this is NA then no weights are used (even if the graph has a weight attribute). By default, weights are ignored. See details for more.
#' @param iter number of iterations during stress optimization
#' @param tol stopping criterion for stress optimization
#' @param mds should an MDS layout be used as initial layout (default: TRUE)
#' @param bbox constrain dimension of output. Only relevant to determine the placement of disconnected graphs
#' @details Be careful when using weights. In most cases, the inverse of the edge weights should be used to ensure that the endpoints of an edges with higher weights are closer together (weights=1/E(g)$weight).
#'
#' This function does not come with direct support for igraph or ggraph.
#'
#' @seealso [layout_constrained_stress]
#' @return matrix of xyz coordinates
#' @references Gansner, E. R., Koren, Y., & North, S. (2004). Graph drawing by stress majorization. *In International Symposium on Graph Drawing* (pp. 239-250). Springer, Berlin, Heidelberg.
#' @export
layout_with_constrained_stress3D <- function(g, coord, fixdim = "x", weights = NA,
                                             iter = 500, tol = 0.0001, mds = TRUE, bbox = 30) {
    ensure_igraph(g)

    oldseed <- get_seed()
    set.seed(42)
    on.exit(restore_seed(oldseed))

    fixdim <- match.arg(fixdim, c("x", "y", "z"))
    fixdim <- ifelse(fixdim == "x", 1, ifelse(fixdim == "y", 2, 3))

    comps <- igraph::components(g, "weak")
    if (comps$no == 1) {
        if (igraph::vcount(g) == 1) {
            return(matrix(c(0, 0, 0), 1, 3))
        } else {
            D <- igraph::distances(g, weights = weights)
            W <- 1 / D^2
            diag(W) <- 0
            n <- igraph::vcount(g)
            xinit <- .init_layout(g, D, mds, n, dim = 3)
            xinit[, fixdim] <- coord
            return(constrained_stress_major3D(xinit, fixdim, W, D, iter, tol))
        }
    } else {
        return(.component_layouter(
            g = g, weights = weights, comps = comps, dim = 3, mds = mds,
            bbox = bbox, iter = iter, tol = tol, FUN = constrained_stress_major3D, fixdim = fixdim, coord = coord
        ))
    }
}

#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#' Layout with fixed coordinates
#'
#' @name layout_fixed_coords
#' @description force-directed graph layout based on stress majorization with
#' fixed coordinates for some nodes
#' @param g igraph object
#' @param coords numeric n x 2 matrix, where n is the number of nodes. values
#' are either NA or fixed coordinates. coordinates are only calculated for the
#' NA values.
#' @param weights possibly a numeric vector with edge weights. If this is NULL and the graph has a weight edge attribute, then the attribute is used. If this is NA then no weights are used (even if the graph has a weight attribute). By default, weights are ignored. See details for more.
#' @param iter number of iterations during stress optimization
#' @param tol stopping criterion for stress optimization
#' @param mds should an MDS layout be used as initial layout (default: TRUE)
#' @param bbox constrain dimension of output. Only relevant to determine the placement of disconnected graphs
#' @details Be careful when using weights. In most cases, the inverse of the edge weights should be used to ensure that the endpoints of an edges with higher weights are closer together (weights=1/E(g)$weight).
#'
#' The layout_igraph_* function should not be used directly. It is only used as an argument for plotting with 'igraph'.
#' 'ggraph' natively supports the layout.
#' @seealso [layout_constrained_stress]
#' @return matrix of xy coordinates
#' @examples
#' library(igraph)
#' set.seed(12)
#' g <- sample_bipartite(10, 5, "gnp", 0.5)
#' fxy <- cbind(c(rep(0, 10), rep(1, 5)), NA)
#' xy <- layout_with_fixed_coords(g, fxy)
#' @export
layout_with_fixed_coords <- function(g, coords, weights = NA,
                                     iter = 500, tol = 0.0001, mds = TRUE, bbox = 30) {
    ensure_igraph(g)

    oldseed <- get_seed()
    set.seed(42) # stress is deterministic and produces same result up to translation. This keeps the layout fixed
    on.exit(restore_seed(oldseed))

    if (missing(coords)) {
        stop('"coords" is missing with no default.')
    }
    if (nrow(coords) != igraph::vcount(g) || ncol(coords) != 2) {
        stop("coords has the wrong dimensions")
    }
    if (all(!is.na(coords))) {
        warning("all coordinates fixed")
        return(coords)
    }
    comps <- igraph::components(g, "weak")
    if (comps$no == 1) {
        if (igraph::vcount(g) == 1) {
            return(matrix(c(0, 0), 1, 2))
        } else {
            D <- igraph::distances(g, weights = weights)
            W <- 1 / D^2
            diag(W) <- 0
            n <- igraph::vcount(g)
            xinit <- .init_layout(g, D, mds, n, dim = 2)
            not_na_idx <- which(!is.na(coords), arr.ind = TRUE)
            xinit[not_na_idx] <- coords[not_na_idx]
            return(fixed_stress_major(xinit, coords, W, D, iter, tol))
        }
    } else {
        # return(.component_layouter(
        #     g = g, weights = weights, comps = comps, dim = 2, mds = mds,
        #     bbox = bbox, iter = iter, tol = tol, FUN = constrained_stress_major, fixdim = fixdim, coord = coord
        # ))
        stop("only connected graphs are supported")
    }
}

#' radial focus group layout
#'
#' @description arrange nodes in concentric circles around a focal node according to their distance from the focus and keep predefined groups in the same angle range.
#'
#' @name layout_focus_group
#' @param g igraph object
#' @param v id of focal node to be placed in the center
#' @param group vector indicating grouping of nodes
#' @param shrink shrink the reserved angle range for a group to increase the gaps between groups
#' @param weights possibly a numeric vector with edge weights. If this is NULL and the graph has a weight edge attribute, then the attribute is used. If this is NA then no weights are used (even if the graph has a weight attribute). By default, weights are ignored. See details for more.
#' @param iter number of iterations during stress optimization
#' @param tol stopping criterion for stress optimization
#' @details Be careful when using weights. In most cases, the inverse of the edge weights should be used to ensure that the endpoints of an edges with higher weights are closer together (weights=1/E(g)$weight).
#' @seealso [layout_focus]
#' The layout_igraph_* function should not be used directly. It is only used as an argument for plotting with 'igraph'.
#' @return matrix of xy coordinates
#' @examples
#' library(igraph)
#' g <- sample_islands(4, 5, 0.8, 2)
#' grp <- as.character(rep(1:4, each = 5))
#' layout_with_focus_group(g, v = 1, group = grp, shrink = 10)
#' @export
layout_with_focus_group <- function(g, v, group, shrink = 10, weights = NA, iter = 500, tol = 0.0001) {
    ensure_igraph(g)
    ensure_connected(g)

    n_grp <- length(unique(group))
    xy <- layout_with_focus(g, v)$xy
    ints <- seq(0, 360, length.out = n_grp + 1)

    for (i in seq_len(n_grp)) {
        xy[group == i, ] <- map_to_angle_range(xy[group == i, ], c(ints[i] + shrink, ints[i + 1] - shrink))
    }
    return(xy)
}

#------------------------------------------------------------------------------#
#------------------------------------------------------------------------------#
#' radial centrality group layout
#'
#' @description arranges nodes in concentric circles according to a centrality index and keeping groups within a angle range
#'
#' @name layout_centrality_group
#' @param g igraph object
#' @param cent centrality scores
#' @param group vector indicating grouping of nodes
#' @param shrink shrink the reserved angle range for a group to increase the gaps between groups
#' @param ... additional arguments to layout_with_centrality
#' The layout_igraph_* function should not be used directly. It is only used as an argument for plotting with 'igraph'.
#' 'ggraph' natively supports the layout.
#' @seealso [layout_centrality]
#' @return matrix of xy coordinates
#' @examples
#' library(igraph)
#' @export
layout_with_centrality_group <- function(g, cent, group, shrink = 10, ...) {
    ensure_igraph(g)
    ensure_connected(g)
    if (missing(group)) {
        stop('argument "group" is missing with no default.')
    }
    if (missing(cent)) {
        stop('argument "group" is missing with no default.')
    }
    n_grp <- length(unique(group))
    xy <- layout_with_centrality(g, cent, ...)
    ints <- seq(0, 360, length.out = n_grp + 1)

    for (i in seq_len(n_grp)) {
        xy[group == i, ] <- map_to_angle_range(xy[group == i, ], c(ints[i] + shrink, ints[i + 1] - shrink))
    }
    return(xy)
}

#' @useDynLib graphlayouts, .registration = TRUE
#' @importFrom Rcpp sourceCpp
NULL

test_that("it works on directed graph with 2 components, in the presence of weights", {
    adj <- matrix(
        c(
            0, 2, 0, 0, 0,
            0, 0, 0, 0, 0,
            0, 0, 0, 3, 1,
            0, 0, 1, 0, 1,
            0, 0, 0, 0, 0
        ),
        nrow = 5
    )
    g <- igraph::graph_from_adjacency_matrix(adj, weighted = TRUE)
    expect_silent(
        r <- layout_with_stress(g, weights = igraph::E(g)$weight)
    )
    expect_is(r, "matrix")
})


test_that("it works on directed connected graph", {
    g <- igraph::make_graph(~ a - +b + -+c - +d:e:f)
    expect_silent(
        r <- layout_with_stress(g)
    )
    expect_is(r, "matrix")
})



test_that("it works on undirected connected graph", {
    g <- igraph::make_graph(~ a - -b - -c - -d:e:f)
    expect_silent(
        r <- layout_with_stress(g)
    )
    expect_is(r, "matrix")
})

test_that("it works on undirected connected weighted graph", {
    g <- igraph::make_graph(~ a - -b - -c - -d:e:f)
    igraph::E(g)$weight <- c(1, 2, 3, 4, 5)
    expect_silent(
        r <- layout_with_stress(g, weights = NULL)
    )
    expect_is(r, "matrix")
})

test_that("it works on an isolates", {
    g <- igraph::make_graph(~a)
    expect_silent(
        r <- layout_with_stress(g)
    )
    expect_is(r, "matrix")
})

test_that("it works on a graph of 5 isolates", {
    g <- igraph::make_graph(~a, b, c, d, e)
    expect_silent(
        r <- layout_with_stress(g)
    )
    expect_is(r, "matrix")
})

test_that("it works on an undirected graph of two connected dyads", {
    g <- igraph::make_graph(~ a - -b, c - -d)
    expect_silent(
        r <- layout_with_stress(g)
    )
    expect_is(r, "matrix")
})


test_that("it works on an undirected graph of two connected dyads with 5 isolates", {
    g <- igraph::make_graph(~ a - -b, c - -d, e, f, g, h, i)
    expect_silent(
        r <- layout_with_stress(g)
    )
    expect_is(r, "matrix")
})


context("Test layout_with_stress3D() on connected graphs")

test_that("that works on directed graph with 2 components, in the presence of weights", {
    adj <- matrix(
        c(
            0, 2, 0, 0, 0,
            0, 0, 0, 0, 0,
            0, 0, 0, 3, 1,
            0, 0, 1, 0, 1,
            0, 0, 0, 0, 0
        ),
        nrow = 5
    )
    g <- igraph::graph_from_adjacency_matrix(adj, weighted = TRUE)
    expect_silent(
        r <- layout_with_stress3D(g, weights = igraph::E(g)$weight)
    )
    expect_is(r, "matrix")
})

test_that("it works on directed connected graph", {
    g <- igraph::make_graph(~ a - +b + -+c - +d:e:f)
    expect_silent(
        r <- layout_with_stress3D(g)
    )
    expect_is(r, "matrix")
})

test_that("it works on undirected connected weighted graph", {
    g <- igraph::make_graph(~ a - -b - -c - -d:e:f)
    igraph::E(g)$weight <- c(1, 2, 3, 4, 5)
    expect_silent(
        r <- layout_with_stress3D(g, weights = NULL)
    )
    expect_is(r, "matrix")
})

test_that("it works on undirected connected graph", {
    g <- igraph::make_graph(~ a - -b - -c - -d:e:f)
    expect_silent(
        r <- layout_with_stress3D(g)
    )
    expect_is(r, "matrix")
})

test_that("it works on undirected disconnected graph", {
    g <- igraph::graph.full(5) + igraph::graph.full(5)
    expect_silent(
        r <- layout_with_stress(g)
    )
    expect_is(r, "matrix")
})

context("layout_with_stress3D() works on disconnected graphs")

test_that("it works on an isolate", {
    g <- igraph::make_graph(~a)
    expect_silent(
        r <- layout_with_stress(g)
    )
    expect_is(r, "matrix")
})

test_that("it works on a graph of 5 isolates", {
    g <- igraph::make_graph(~a, b, c, d, e)
    expect_silent(
        r <- layout_with_stress3D(g)
    )
    expect_is(r, "matrix")
})

test_that("it works on an undirected graph of two connected dyads", {
    g <- igraph::make_graph(~ a - -b, c - -d)
    expect_silent(
        r <- layout_with_stress3D(g)
    )
    expect_is(r, "matrix")
})


test_that("it works on an undirected graph of two connected dyads with 5 isolates", {
    g <- igraph::make_graph(~ a - -b, c - -d, e, f, g, h, i)
    expect_silent(
        r <- layout_with_stress3D(g)
    )
    expect_is(r, "matrix")
})

test_that("it works on undirected disconnected graph", {
    g <- igraph::graph.full(5) + igraph::graph.full(5)
    expect_silent(
        r <- layout_with_stress3D(g)
    )
    expect_is(r, "matrix")
})

context("Test layout_with_focus() on connected graphs")

test_that("it works on undirected connected graphs", {
    g <- igraph::make_star(5, mode = "undirected", center = 1)
    expect_error(layout_with_focus(g))
    expect_silent(
        r <- layout_with_focus(g, v = 1)$xy
    )
    expect_is(r, "matrix")
})


test_that("it fails for disconnected graphs", {
    expect_error(layout_with_focus(igraph::graph.empty(n = 10, directed = FALSE)))
})

context("Test layout_with_centrality() on connected graphs")

test_that("it works on undirected connected graphs", {
    g <- igraph::make_star(5, mode = "undirected", center = 1)
    expect_error(layout_with_centrality(g))
    expect_silent(
        r <- layout_with_centrality(g, cent = igraph::degree(g))
    )
    expect_is(r, "matrix")
})


test_that("it fails for disconnected graphs", {
    expect_error(layout_with_centrality(igraph::graph.empty(n = 10, directed = FALSE)))
})

context("Test layout_with_constrained_stress()")


test_that("it works on undirected connected graph", {
    g <- igraph::make_graph(~ a - -b - -c - -d:e:f)
    expect_error(layout_with_constrained_stress(g))
    expect_silent(
        r <- layout_with_constrained_stress(g, coord = rep(1, 6))
    )
    expect_is(r, "matrix")
})

test_that("it works on undirected disconnected graph", {
    g <- igraph::make_full_graph(5) + igraph::make_full_graph(5)
    fix <- rep(c(1, 2), each = 5)
    expect_silent(
        r <- layout_with_constrained_stress(g, coord = fix)
    )
    expect_is(r, "matrix")
    expect_true(all(r[, 1] == fix))
})


context("Test layout_with_constrained_stress3D() on connected graphs")
test_that("it works on undirected connected graph", {
    g <- igraph::make_graph(~ a - -b - -c - -d:e:f)
    expect_error(layout_with_constrained_stress3D(g))
    expect_silent(
        r <- layout_with_constrained_stress3D(g, coord = rep(1, 6))
    )
    expect_is(r, "matrix")
})

test_that("it works on undirected disconnected graph", {
    g <- igraph::make_full_graph(5) + igraph::make_full_graph(5)
    fix <- rep(c(1, 2), each = 5)
    expect_silent(
        r <- layout_with_constrained_stress3D(g, coord = fix)
    )
    expect_is(r, "matrix")
    expect_true(all(r[, 1] == fix))
})

context("Test layout_with_*_grouped()")
test_that("grouped layouts work", {
    g <- igraph::graph.full(10)
    grp <- rep(c(1, 2), each = 5)
    expect_is(layout_with_focus_group(g, v = 1, grp), "matrix")

    g <- igraph::graph.star(10)
    grp <- rep(c(1, 2), each = 5)
    expect_is(layout_with_centrality_group(g, cent = igraph::degree(g), grp), "matrix")
})

test_that("test errors in stress layouts", {
    expect_error(layout_with_stress(5))
    expect_error(layout_with_constrained_stress(5))
    expect_error(layout_with_stress3D(5))
    expect_error(layout_with_focus(5))

    expect_error(layout_with_focus(igraph::graph.full(5)))
    expect_error(layout_with_centrality(igraph::graph.full(5)))

    expect_error(layout_with_constrained_stress(igraph::graph.full(5), fixdim = "z"))
    expect_error(layout_with_constrained_stress(igraph::graph.full(5), fixdim = "x"))

    expect_error(layout_with_focus_group(igraph::graph.full(5)))
    expect_error(layout_with_centrality_group(igraph::graph.full(5)))
    expect_error(layout_with_focus_group(igraph::graph.full(5), v = 2))
    expect_error(layout_with_centrality_group(igraph::graph.full(5), cent = igraph::degree(g)))
})

test_that("layout_with_fixed_nodes works", {
    set.seed(12)
    g <- sample_bipartite(10, 5, "gnp", 0.5)
    fxy <- cbind(c(rep(0, 10), rep(1, 5)), NA)
    expect_silent(
        r <- layout_with_fixed_coords(g, fxy)
    )
    expect_is(r, "matrix")
    expect_error(layout_with_fixed_coords(g))
    expect_warning(layout_with_fixed_coords(g, matrix(1, 15, 2)))
    expect_error(layout_with_fixed_coords(g, matrix(1, 12, 2)))
})

# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

constrained_stress <- function(x, W, D) {
    .Call(`_graphlayouts_constrained_stress`, x, W, D)
}

constrained_stress_major <- function(y, dim, W, D, iter, tol) {
    .Call(`_graphlayouts_constrained_stress_major`, y, dim, W, D, iter, tol)
}

fixed_stress_major <- function(y, fixedCoords, W, D, iter, tol) {
    .Call(`_graphlayouts_fixed_stress_major`, y, fixedCoords, W, D, iter, tol)
}

constrained_stress3D <- function(x, W, D) {
    .Call(`_graphlayouts_constrained_stress3D`, x, W, D)
}

constrained_stress_major3D <- function(y, dim, W, D, iter, tol) {
    .Call(`_graphlayouts_constrained_stress_major3D`, y, dim, W, D, iter, tol)
}

criterion_angular_resolution <- function(adj, xy) {
    .Call(`_graphlayouts_criterion_angular_resolution`, adj, xy)
}

criterion_edge_length <- function(el, xy, lg) {
    .Call(`_graphlayouts_criterion_edge_length`, el, xy, lg)
}

criterion_balanced_edge_length <- function(adj_deg2, xy) {
    .Call(`_graphlayouts_criterion_balanced_edge_length`, adj_deg2, xy)
}

criterion_line_straightness <- function() {
    .Call(`_graphlayouts_criterion_line_straightness`)
}

criterion_octilinearity <- function(el, xy) {
    .Call(`_graphlayouts_criterion_octilinearity`, el, xy)
}

layout_as_metro_iter <- function(adj, el, adj_deg2, xy, bbox, l, gr, w, bsize) {
    .Call(`_graphlayouts_layout_as_metro_iter`, adj, el, adj_deg2, xy, bbox, l, gr, w, bsize)
}

reweighting <- function(el, N_ranks) {
    .Call(`_graphlayouts_reweighting`, el, N_ranks)
}

sparseStress <- function(y, D, Rp, pivots, A, maxIter) {
    .Call(`_graphlayouts_sparseStress`, y, D, Rp, pivots, A, maxIter)
}

stress <- function(x, W, D) {
    .Call(`_graphlayouts_stress`, x, W, D)
}

stress_major <- function(y, W, D, iter, tol) {
    .Call(`_graphlayouts_stress_major`, y, W, D, iter, tol)
}

stress_radii <- function(y, W, D, r, tseq) {
    .Call(`_graphlayouts_stress_radii`, y, W, D, r, tseq)
}

stress_focus <- function(y, W, D, Z, tseq, iter, tol) {
    .Call(`_graphlayouts_stress_focus`, y, W, D, Z, tseq, iter, tol)
}

stress3D <- function(x, W, D) {
    .Call(`_graphlayouts_stress3D`, x, W, D)
}

stress_major3D <- function(y, W, D, iter, tol) {
    .Call(`_graphlayouts_stress_major3D`, y, W, D, iter, tol)
}

#' spectral graph layouts
#' @description Using a set of eigenvectors of matrices associated with a graph as coordinates
#' @name layout_spectral
#' @param g igraph object
#' @param type matrix to be used for spectral decomposition. either 'adjacency' or 'laplacian'
#' @param ev eigenvectors to be used. Either 'smallest' or 'largest'.
#' @details The layout_igraph_* function should not be used directly. It is only used as an argument for plotting with 'igraph'.
#' 'ggraph' natively supports the layout.
#' @author David Schoch
#' @return matrix of xy coordinates
#' @examples
#' library(igraph)
#'
#' g <- sample_gnp(50, 0.2)
#'
#' xy <- layout_with_eigen(g, type = "adjacency", ev = "largest")
#'
#' xy <- layout_with_eigen(g, type = "adjacency", ev = "smallest")
#'
#' xy <- layout_with_eigen(g, type = "laplacian", ev = "largest")
#'
#' xy <- layout_with_eigen(g, type = "laplacian", ev = "smallest")
#' @export

layout_with_eigen <- function(g, type = "laplacian", ev = "smallest") {
    ensure_igraph(g)
    if (!igraph::is_connected(g)) {
        stop("g must be connected")
    }
    if (igraph::is_directed(g)) {
        warning("g is directed. undirected version is used for the layout.")
        g <- igraph::as.undirected(g)
    }
    if (!type %in% c("laplacian", "adjacency")) {
        stop("type must be one of 'laplacian' or 'adjacency'")
    }
    if (!ev %in% c("largest", "smallest")) {
        stop("ev must be one of 'smallest' or 'largest'")
    }
    n <- igraph::vcount(g)
    if (type == "adjacency") {
        A <- igraph::as_adj(g, type = "both")
    } else {
        A <- igraph::laplacian_matrix(g)
    }
    sA <- eigen(A)
    if (ev == "largest") {
        xy <- sA$vectors[, 1:2]
    } else if (ev == "smallest" && type == "adjacency") {
        xy <- sA$vectors[, (n - 1):n]
    } else {
        xy <- sA$vectors[, (n - 2):(n - 1)]
    }
    xy
}

library(igraph)
test_that("layout_spectral works", {
  g <- igraph::make_graph( ~ a -- b -- c -- d:e:f)

  expect_is(layout_with_eigen(g,type = "adjacency",ev = "largest"),"matrix")
  expect_is(layout_with_eigen(g,type = "adjacency",ev = "smallest"),"matrix")
  expect_is(layout_with_eigen(g,type = "laplacian",ev = "largest"),"matrix")
  expect_is(layout_with_eigen(g,type = "laplacian",ev = "smallest"),"matrix")

  expect_warning(layout_with_eigen(igraph::as.directed(g)))
  expect_error(layout_with_eigen(igraph::graph.empty(10,directed = FALSE)))

  expect_error(layout_with_eigen(g, type = "laplacian", ev = "middlest"))
  expect_error(layout_with_eigen(g, type = "random"))
  expect_error(layout_with_eigen(1))
})

#' multilevel layout
#' @description Layout algorithm to visualize multilevel networks
#' @name layout_multilevel
#' @param g igraph object. Must have a vertex attribute "lvl" which is 1 or 2.
#' @param type one of "all", "separate","fix1" or "fix2". see details
#' @param FUN1 if type="separate", the layout function to be used for level 1
#' @param FUN2 if type="separate", the layout function to be used for level 2
#' @param params1 named list of parameters for FUN1
#' @param params2 named list of parameters for FUN2
#' @param ignore_iso treatment of isolates within levels. see details
#' @param project2D logical. Defaults to TRUE (project to 2D).
#' @param alpha angle for isometric projection between 0 and 90
#' @param beta  angle for isometric projection between 0 and 90
#' @details
#' The algorithm internally computes a 3D layout where each level is in a separate y-plane.
#' The layout is then projected into 2D via an isometric mapping, controlled by the parameters
#' `alpha` and `beta`. It may take some adjusting to `alpha` and `beta` to find a good perspective.
#'
#' If type="all", the layout is computed at once for the complete network.
#' For type="separate", two user specified layout algorithms (`FUN1` and `FUN2`) are used for the levels.
#' The named lists `param1` and `param2` can be used to set parameters for `FUN1` and `FUN2`.
#' This option helpful for situations where different structural features of the levels should be emphasized.
#'
#' For type="fix1" and type="fix2" only one of the level layouts is fixed. The other one is calculated by optimizing the
#' inter level ties, such that they are drawn (almost) vertical.
#'
#' The `ignore_iso` parameter controls the handling of isolates. If TRUE, nodes without inter level edges are ignored during the layout process
#' and added at the end. If FALSE they are left unchanged
#'
#' The layout_igraph_* function should not be used directly. It is only used as an argument for plotting with 'igraph'.
#' @return matrix of xy coordinates
#' @examples
#' library(igraph)
#' data("multilvl_ex")
#'\dontrun{
#' # compute a layout for the whole network
#' xy <- layout_as_multilevel(multilvl_ex, type = "all", alpha = 25, beta = 45)
#'
#' # compute a layout for each level separately and combine them
#' xy <- layout_as_multilevel(multilvl_ex,
#'     type = "separate",
#'     FUN1 = layout_as_backbone,
#'     FUN2 = layout_with_stress,
#'     alpha = 25, beta = 45
#' )
#' }
#' @export
layout_as_multilevel <- function(
  g,
  type = "all",
  FUN1,
  FUN2,
  params1 = NULL,
  params2 = NULL,
  ignore_iso = TRUE,
  project2D = TRUE,
  alpha = 35,
  beta = 45
) {
  type <- match.arg(type, c("all", "separate", "fix1", "fix2"))

  if (!"lvl" %in% igraph::vertex_attr_names(g)) {
    stop(
      "level information should be stored in a vertex attribute called 'lvl'"
    )
  }
  # 3D stress
  if (type == "all") {
    xyz <- layout_with_constrained_stress3D(
      g,
      coord = igraph::V(g)$lvl,
      fixdim = "y"
    )
    xyz <- optim_rotation(g, xyz)
    xyz <- optim_isolates(g, xyz)
    xyz[, c(1, 3)] <- c(
      normalise(xyz[, 1], to = c(1, 2)),
      normalise(xyz[, 3], to = c(1, 2))
    )
    # separate
  } else if (type == "separate") {
    if (missing(FUN1) || missing(FUN2)) {
      stop("FUN1 and FUN2 must both be specified")
    }

    lvl1 <- which(igraph::V(g)$lvl == 1)
    lvl2 <- which(igraph::V(g)$lvl == 2)

    g1 <- igraph::induced_subgraph(g, lvl1)
    g2 <- igraph::induced_subgraph(g, lvl2)
    if (ignore_iso) {
      iso1 <- which(igraph::degree(g1) == 0)
      iso2 <- which(igraph::degree(g2) == 0)
      g1 <- igraph::delete_vertices(g1, iso1)
      g2 <- igraph::delete_vertices(g2, iso2)
    }

    if (is.null(params1)) {
      xy1 <- FUN1(g1)
    } else {
      if (!all(names(params1) %in% names(formals(FUN1)))) {
        stop("params1 contains invalid parameters.")
      }
      formals(FUN1)[names(params1)] <- params1
      xy1 <- FUN1(g1)
    }
    if (typeof(xy1) == "list") {
      xy1 <- xy1$xy
    }

    if (is.null(params2)) {
      xy2 <- FUN2(g2)
    } else {
      if (!all(names(params2) %in% names(formals(FUN2)))) {
        stop("params2 contains invalid parameters.")
      }
      formals(FUN2)[names(params2)] <- params2
      xy2 <- FUN2(g2)
    }
    if (typeof(xy2) == "list") {
      xy2 <- xy2$xy
    }
    xyz <- cbind(0, igraph::V(g)$lvl, 0)
    if (ignore_iso) {
      if (length(iso1) != 0) {
        xy1_tmp <- matrix(0, length(lvl1), 2)
        xy1_tmp[-iso1, ] <- xy1
        xy1 <- xy1_tmp
      }
      if (length(iso2) != 0) {
        xy2_tmp <- matrix(0, length(lvl2), 2)
        xy2_tmp[-iso2, ] <- xy2
        xy2 <- xy2_tmp
      }
    }
    xy1[, 1] <- normalise(xy1[, 1], to = c(1, 2))
    xy1[, 2] <- normalise(xy1[, 2], to = c(1, 2))
    xy2[, 1] <- normalise(xy2[, 1], to = c(1, 2))
    xy2[, 2] <- normalise(xy2[, 2], to = c(1, 2))
    xyz[lvl1, c(1, 3)] <- xy1
    xyz[lvl2, c(1, 3)] <- xy2
    xyz <- optim_rotation(g, xyz)
    xyz <- optim_isolates(g, xyz)
    # fix level 1
  } else if (type == "fix1") {
    if (missing(FUN1)) {
      stop("FUN1 must must be specified")
    }
    lvl1 <- which(igraph::V(g)$lvl == 1)
    lvl2 <- which(igraph::V(g)$lvl == 2)
    g1 <- igraph::induced_subgraph(g, lvl1)

    if (ignore_iso) {
      iso1 <- which(igraph::degree(g1) == 0)
      g1 <- igraph::delete_vertices(g1, iso1)
    }

    if (is.null(params1)) {
      xy1 <- FUN1(g1)
    } else {
      if (!all(names(params1 %in% names(formals(FUN1))))) {
        stop("params1 contains invalid parameters.")
      }
      formals(FUN1)[names(params1)] <- params1
      xy1 <- FUN1(g1)
    }
    if (typeof(xy1) == "list") {
      xy1 <- xy1$xy
    }
    xyz <- cbind(0, igraph::V(g)$lvl, 0)
    if (ignore_iso) {
      if (length(iso1) != 0) {
        xy1_tmp <- matrix(0, length(lvl1), 2)

        mx <- mean(xy1[, 1], na.rm = TRUE)
        my <- mean(xy1[, 2], na.rm = TRUE)
        r <- max(sqrt((xy1[, 1] - mx)^2 + (xy1[, 2] - my)^2))
        isox <- stats::runif(length(iso1), mx - r, mx + r)
        isoy <- sample(c(-1, 1), length(iso1), replace = T) *
          sqrt(r^2 - (isox - mx)^2) +
          my
        xy1_tmp[-iso1, ] <- xy1
        xy1_tmp[iso1, ] <- cbind(isox, isoy)
        xy1 <- xy1_tmp
      }
    }
    xy1[, 1] <- normalise(xy1[, 1], to = c(1, 2))
    xy1[, 2] <- normalise(xy1[, 2], to = c(1, 2))
    xy2 <- optim_level(g, 1, xy1)
    xyz[lvl1, c(1, 3)] <- xy1
    xyz[lvl2, c(1, 3)] <- xy2
    # fix level 2
  } else if (type == "fix2") {
    if (missing(FUN2)) {
      stop("FUN2 must must be specified")
    }
    lvl1 <- which(igraph::V(g)$lvl == 1)
    lvl2 <- which(igraph::V(g)$lvl == 2)
    g2 <- igraph::induced_subgraph(g, lvl2)

    if (ignore_iso) {
      iso2 <- which(igraph::degree(g2) == 0)
      g2 <- igraph::delete_vertices(g2, iso2)
    }

    if (is.null(params2)) {
      xy2 <- FUN2(g2)
    } else {
      if (!all(names(params2 %in% names(formals(FUN2))))) {
        stop("params1 contains invalid parameters.")
      }
      formals(FUN2)[names(params2)] <- params2
      xy2 <- FUN2(g2)
    }
    if (typeof(xy2) == "list") {
      xy2 <- xy2$xy
    }
    xyz <- cbind(0, igraph::V(g)$lvl, 0)

    if (ignore_iso) {
      if (length(iso2) != 0) {
        xy2_tmp <- matrix(0, length(lvl2), 2)

        mx <- mean(xy2[, 1], na.rm = TRUE)
        my <- mean(xy2[, 2], na.rm = TRUE)
        r <- max(sqrt((xy2[, 1] - mx)^2 + (xy2[, 2] - my)^2))

        isox <- stats::runif(length(iso2), mx - r, mx + r)
        isoy <- sample(c(-1, 1), length(iso2), replace = T) *
          sqrt(r^2 - (isox - mx)^2) +
          my
        xy2_tmp[-iso2, ] <- xy2
        xy2_tmp[iso2, ] <- cbind(isox, isoy)
        xy2 <- xy2_tmp
      }
    }

    xy2[, 1] <- normalise(xy2[, 1], to = c(1, 2))
    xy2[, 2] <- normalise(xy2[, 2], to = c(1, 2))
    xy1 <- optim_level(g, 2, xy2)
    xyz[lvl1, c(1, 3)] <- xy1
    xyz[lvl2, c(1, 3)] <- xy2
  }
  if (project2D) {
    xy <- iso_project(xyz, a = alpha, b = beta)
    return(xy)
  } else {
    return(xyz)
  }
}

test_that("layout_mulitlevel works", {
  testthat::skip_if_not_installed("oaqc")
  data("multilvl_ex")
  expect_is(
    layout_as_multilevel(multilvl_ex, type = "all", alpha = 25, beta = 45),
    "matrix"
  )
  expect_is(
    layout_as_multilevel(
      multilvl_ex,
      type = "separate",
      FUN1 = layout_as_backbone,
      FUN2 = layout_with_stress,
      alpha = 25,
      beta = 45
    ),
    "matrix"
  )
  expect_is(
    layout_as_multilevel(
      multilvl_ex,
      type = "fix2",
      FUN2 = layout_with_stress,
      alpha = 25,
      beta = 45
    ),
    "matrix"
  )
  expect_is(
    layout_as_multilevel(
      multilvl_ex,
      type = "fix1",
      FUN1 = layout_with_stress,
      alpha = 25,
      beta = 45
    ),
    "matrix"
  )
  expect_error(layout_as_multilevel(igraph::make_full_graph(10)))

  expect_error(layout_as_multilevel(multilvl_ex, type = "fix1"))
  expect_error(layout_as_multilevel(multilvl_ex, type = "fix2"))
  expect_error(layout_as_multilevel(multilvl_ex, type = "fix3"))

  g <- igraph::delete_vertex_attr(multilvl_ex, "lvl")
  expect_error(layout_as_multilevel(g, type = "fix2"))

  expect_error(layout_as_multilevel(multilvl_ex, type = "separate"))
  expect_error(layout_as_multilevel(
    multilvl_ex,
    type = "separate",
    FUN1 = layout_as_backbone,
    params1 = list(wrong = "a"),
    FUN2 = layout_with_stress
  ))
  expect_error(layout_as_multilevel(
    multilvl_ex,
    type = "separate",
    FUN1 = layout_as_backbone,
    FUN2 = layout_with_stress,
    params2 = list(wrong = "a")
  ))
  expect_no_error(layout_as_multilevel(
    multilvl_ex,
    type = "separate",
    FUN1 = layout_as_backbone,
    params1 = list(keep = 0.3),
    FUN2 = layout_with_stress,
    params2 = list(bbox = 15)
  ))
})

#' pivot MDS graph layout
#'
#' @name layout_pmds
#' @description similar to \link[igraph]{layout_with_mds} but uses only a small set of pivots for MDS. Considerably faster than MDS and thus applicable for larger graphs.
#' @param g igraph object
#' @param pivots number of pivots
#' @param D precomputed distances from pivots to all nodes (if available, default: NULL)
#' @param dim dimensionality of layout (defaults to 2)
#' @param weights possibly a numeric vector with edge weights. If this is NULL and the graph has a weight edge attribute, then the attribute is used. If this is NA then no weights are used (even if the graph has a weight attribute). By default, weights are ignored. See details for more.
#' @details Be careful when using weights. In most cases, the inverse of the edge weights should be used to ensure that the endpoints of an edges with higher weights are closer together (weights=1/E(g)$weight)
#'
#' The layout_igraph_* function should not be used directly. It is only used as an argument for plotting with 'igraph'.
#' 'ggraph' natively supports the layout.
#' @author David Schoch
#' @return matrix of coordinates
#' @references Brandes, U. and Pich, C. (2006). Eigensolver Methods for Progressive Multidimensional Scaling of Large Data. In *International Symposium on Graph Drawing* (pp. 42-53). Springer
#' @examples
#' \dontrun{
#' library(igraph)
#' library(ggraph)
#'
#' g <- sample_gnp(1000, 0.01)
#'
#' xy <- layout_with_pmds(g, pivots = 100)
#' }
#' @export
layout_with_pmds <- function(g, pivots, weights = NA, D = NULL, dim = 2) {
    ensure_igraph(g)
    ensure_connected(g)

    if (missing(pivots) && is.null(D)) {
        stop('argument "pivots" is missing, with no default.')
    }
    if (!missing(pivots)) {
        if (pivots > igraph::vcount(g)) {
            stop('"pivots" must be less than the number of nodes in the graph.')
        }
    }
    if (is.null(D)) {
        pivs <- sample(1:igraph::vcount(g), pivots)
        D <- t(igraph::distances(g, v = pivs, weights = weights))
    }
    cmean <- colMeans(D^2)
    rmean <- rowMeans(D^2)
    Dmat <- D^2 - outer(rmean, cmean, function(x, y) x + y) + mean(D^2)
    sl2 <- svd(Dmat)

    xy <- (Dmat %*% sl2$v[, 1:dim])
    xy
}


#' sparse stress graph layout
#'
#' @name layout_sparse_stress
#' @description stress majorization for larger graphs based on a set of pivot nodes.
#' @param g igraph object
#' @param pivots number of pivots
#' @param weights ignored
#' @param iter number of iterations during stress optimization
#' @details The layout_igraph_* function should not be used directly. It is only used as an argument for plotting with 'igraph'.
#' 'ggraph' natively supports the layout.
#' @author David Schoch
#' @return matrix of xy coordinates
#' @references Ortmann, M. and Klimenta, M. and Brandes, U. (2016). A Sparse Stress Model. https://arxiv.org/pdf/1608.08909.pdf
#' @examples
#' \dontrun{
#' library(igraph)
#' library(ggraph)
#'
#' g <- sample_gnp(1000, 0.005)
#'
#' ggraph(g, layout = "sparse_stress", pivots = 100) +
#'     geom_edge_link0(edge_colour = "grey66") +
#'     geom_node_point(shape = 21, fill = "grey25", size = 5) +
#'     theme_graph()
#' }
#' @export

layout_with_sparse_stress <- function(g, pivots, weights = NA, iter = 500) {
    ensure_igraph(g)
    ensure_connected(g)
    if (!all(is.na(weights))) {
        warning("weights are not supported. unweighted graph is used instead.")
    }
    if (missing(pivots)) {
        stop('argument "pivots" is missing, with no default.')
    }
    if (pivots > igraph::vcount(g)) {
        stop('"pivots" must be less than the number of nodes in the graph.')
    }
    comps <- igraph::components(g)
    if (comps$no == 1) {
        prep <- .sparse_prepare(g, pivots)
        A <- igraph::as_adj(g, type = "both", sparse = TRUE)
        return(sparseStress(prep$y, prep$D, prep$RpL, prep$pivs, A, iter))
    } else {
        # TBD
    }
}

.sparse_prepare <- function(g, pivots) {
    pivs <- sample(1:igraph::vcount(g), pivots)

    D <- t(igraph::distances(g, v = pivs, weights = NA))
    Rp <- apply(D, 1, which.min)
    y <- layout_with_pmds(g, pivots, D = D, weights = NA)

    # rescale
    el <- igraph::as_edgelist(g, names = FALSE)
    norm1 <- sum(sqrt((y[el[, 1], 1] - y[el[, 2], 1])^2 + (y[el[, 1], 2] - y[el[, 2], 2])^2))

    y <- y * (igraph::ecount(g) / norm1)

    RpL <- lapply(seq_along(pivs), function(x) which(Rp == x) - 1)
    pivs <- pivs - 1
    list(RpL = RpL, pivs = pivs, y = y, D = D)
}

test_that("layout_with_pmds works", {
    g <- igraph::make_graph(~ a - -b - -c - -d:e:f)
    expect_is(layout_with_pmds(g, 5), "matrix")
    expect_equal(ncol(layout_with_pmds(g, 5, dim = 3)), 3)
    expect_error(layout_with_pmds(g))
    expect_error(layout_with_pmds(g, 10))
    expect_error(layout_with_pmds(1))
    expect_no_error(layout_with_pmds(g, pivots = 5, weights = rep(4, 5)))

    g <- igraph::make_full_graph(10) + igraph::make_full_graph(10)
    expect_error(layout_with_pmds(g, 10))
})

test_that("layout_with_sparse_stress works", {
    g <- igraph::make_graph(~ a - -b - -c - -d:e:f)
    expect_is(layout_with_sparse_stress(g, 5), "matrix")
    expect_error(layout_with_sparse_stress(g))
    expect_error(layout_with_sparse_stress(g, 10))
    expect_error(layout_with_sparse_stress(1))

    g <- igraph::make_full_graph(10) + igraph::make_full_graph(10)
    expect_error(layout_with_sparse_stress(g, 10))
    g <- igraph::make_full_graph(10)
    expect_warning(layout_with_sparse_stress(g, pivots = 5, weights = rep(4, 45)))
})

#' backbone graph layout
#' @description emphasizes a hidden group structure if it exists in the graph. Calculates a layout for a sparsified network only including the most embedded edges. Deleted edges are added back after the layout is calculated.
#' @name layout_backbone
#' @param g igraph object
#' @param keep fraction of edges to keep during backbone calculation
#' @param backbone logical. Return edge ids of the backbone (Default: TRUE)
#' @details
#' The layout_igraph_* function should not be used directly. It is only used as an argument for plotting with 'igraph'.
#' 'ggraph' natively supports the layout.
#' @return list of xy coordinates and vector of edge ids included in the backbone
#' @examples
#' library(igraph)
#' \dontrun{
#' g <- sample_islands(9, 20, 0.4, 9)
#' g <- simplify(g)
#' V(g)$grp <- as.character(rep(1:9, each = 20))
#' bb <- layout_as_backbone(g, keep = 0.4)
#'
#' # add backbone links as edge attribute
#' E(g)$col <- FALSE
#' E(g)$col[bb$backbone] <- TRUE
#' }
#' @references Nocaj, A., Ortmann, M., & Brandes, U. (2015). Untangling the hairballs of multi-centered, small-world online social media networks. Journal of Graph Algorithms and Applications: JGAA, 19(2), 595-618.
#' @export
#'

layout_as_backbone <- function(g, keep = 0.2, backbone = TRUE) {
    ensure_igraph(g)
    if (!requireNamespace("oaqc", quietly = TRUE)) {
        stop("oaqc is needed for this function to work. Please install it.", call. = FALSE)
    }
    if (igraph::ecount(g) == 0) {
        stop("graph is empty")
    }
    if (igraph::any_multiple(g)) {
        stop("backbone layout does not work with multiple edges.")
    }
    if (igraph::is_directed(g)) {
        stop("backbone layout does not work with directed edges.")
    }
    if (igraph::any_loop(g)) {
        stop("backbone layout does not work with loops.")
    }

    if (any(igraph::components(g)$no > 1)) {
        warning("input graph is disconnected. The algorithm works best on connected graphs and may lead to misleading results for graphs with disconnected components. Run the algorithm on each component separately and delete isolated nodes to mitigate this issue.")
    }

    # weighting ----
    orbs <- oaqc::oaqc(igraph::as_edgelist(g, names = FALSE) - 1, non_ind_freq = TRUE)
    e11 <- orbs$e_orbits_non_ind[, 11]

    qu <- rep(0, igraph::vcount(g))
    el <- igraph::as_edgelist(g, names = FALSE)
    el <- cbind(el, e11)
    for (e in seq_len(nrow(el))) {
        qu[el[e, 1]] <- qu[el[e, 1]] + el[e, 3]
        qu[el[e, 2]] <- qu[el[e, 2]] + el[e, 3]
    }
    w <- apply(el, 1, function(x) x[3] / sqrt(qu[x[1]] * qu[x[2]]))

    w[is.na(w)] <- 0
    w[is.infinite(w)] <- 0
    igraph::E(g)$weight <- w

    # reweighting -----
    w <- max_prexif_jaccard(g)
    igraph::E(g)$weight <- w

    # umst ----
    g_umst <- umst(g)

    # filtering ----
    igraph::E(g)$bone <- w >= sort(w, decreasing = TRUE)[ceiling(igraph::ecount(g) * keep)]
    g_bone <- igraph::graph_from_edgelist(el[igraph::E(g)$bone, 1:2], directed = F)
    g_lay <- igraph::simplify(igraph::union(g_umst, g_bone))
    # if there is an issue with isolates (see #44)
    if (igraph::vcount(g_lay) != igraph::vcount(g)) {
        n_iso <- igraph::vcount(g) - igraph::vcount(g_lay)
        g_lay <- igraph::add_vertices(g_lay, n_iso)
    }
    if (backbone) {
        bb <- backbone_edges(g, g_lay)
    } else {
        bb <- NULL
    }
    xy <- layout_with_stress(g_lay)
    list(xy = xy, backbone = bb)
}

#-------------------------------------------------------------------------------
# helper functions
#-------------------------------------------------------------------------------

umst <- function(g) {
    el <- igraph::as_edgelist(g, names = FALSE)
    el <- cbind(el, igraph::E(g)$weight)
    el <- el[order(el[, 3], decreasing = TRUE), ]
    el <- cbind(el, rank(-el[, 3]))
    vfind <- 1:igraph::vcount(g)
    el_un <- matrix(0, 0, 2)
    for (i in unique(el[, 4])) {
        el_tmp <- matrix(0, 0, 2)
        Bi <- which(el[, 4] == i)
        for (e in Bi) {
            u <- el[e, 1]
            v <- el[e, 2]
            if (vfind[u] != vfind[v]) {
                el_tmp <- rbind(el_tmp, c(u, v))
            }
        }
        if (nrow(el_tmp) == 0) {
            next()
        }
        for (eb in seq_len(nrow(el_tmp))) {
            u <- el_tmp[eb, 1]
            v <- el_tmp[eb, 2]
            partu <- vfind[u]
            partv <- vfind[v]
            vfind[v] <- partu
            if (any(vfind == partv)) {
                vfind[vfind == partv] <- partu
            }
        }
        el_un <- rbind(el_un, el_tmp)
    }
    return(igraph::simplify(igraph::graph_from_edgelist(el_un, directed = FALSE)))
}


backbone_edges <- function(g, g_lay) {
    tmp <- rbind(igraph::as_edgelist(g_lay), igraph::as_edgelist(g, names = FALSE))
    which(duplicated(tmp)) - igraph::ecount(g_lay)
}

max_prexif_jaccard <- function(g) {
    if ("name" %in% igraph::vertex_attr_names(g)) {
        g <- igraph::delete_vertex_attr(g, "name")
    }
    el_tbl <- igraph::as_data_frame(g, "edges")

    N_ranks <- lapply(1:igraph::vcount(g), get_rank, el_tbl = el_tbl)
    el <- igraph::as_edgelist(g, names = FALSE)
    new_w <- reweighting(el - 1, N_ranks)
    new_w
}

get_rank <- function(el_tbl, u) {
    Nu_idx <- el_tbl[["from"]] == u | el_tbl[["to"]] == u
    omega <- el_tbl[Nu_idx, "weight"]
    Nu <- setdiff(c(el_tbl[Nu_idx, "from"], el_tbl[Nu_idx, "to"]), u)
    r <- rank(-omega)
    r <- match(r, sort(unique(r))) - 1
    Nru <- cbind(Nu - 1, r)
    Nru[order(Nru[, 2]), , drop = FALSE]
}

test_that("backbone layout works", {
  testthat::skip_if_not_installed("oaqc")
  xy <- layout_as_backbone(igraph::make_full_graph(10))
  expect_is(xy$xy, "matrix")
  expect_is(xy$backbone, "numeric")
  expect_error(layout_as_backbone(5))
  expect_error(layout_as_backbone(igraph::make_empty_graph(5)))
  expect_error(layout_as_backbone(igraph::graph_from_adjacency_matrix(
    matrix(c(0, 2, 2, 0), 2, 2),
    weighted = NULL,
    mode = "undirected"
  )))
  expect_error(layout_as_backbone(igraph::make_full_graph(5, directed = TRUE)))
  expect_warning(layout_as_backbone(
    igraph::make_full_graph(5) + igraph::make_full_graph(5)
  ))
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_spectral.R, R/layouts.R
\name{layout_spectral}
\alias{layout_spectral}
\alias{layout_with_eigen}
\alias{layout_igraph_eigen}
\title{spectral graph layouts}
\usage{
layout_with_eigen(g, type = "laplacian", ev = "smallest")

layout_igraph_eigen(g, type = "laplacian", ev = "smallest", circular)
}
\arguments{
\item{g}{igraph object}

\item{type}{matrix to be used for spectral decomposition. either 'adjacency' or 'laplacian'}

\item{ev}{eigenvectors to be used. Either 'smallest' or 'largest'.}

\item{circular}{not used}
}
\value{
matrix of xy coordinates
}
\description{
Using a set of eigenvectors of matrices associated with a graph as coordinates
}
\details{
The layout_igraph_* function should not be used directly. It is only used as an argument for plotting with 'igraph'.
'ggraph' natively supports the layout.
}
\examples{
library(igraph)

g <- sample_gnp(50, 0.2)

xy <- layout_with_eigen(g, type = "adjacency", ev = "largest")

xy <- layout_with_eigen(g, type = "adjacency", ev = "smallest")

xy <- layout_with_eigen(g, type = "laplacian", ev = "largest")

xy <- layout_with_eigen(g, type = "laplacian", ev = "smallest")
}
\author{
David Schoch
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_umap.R, R/layouts.R
\name{layout_umap}
\alias{layout_umap}
\alias{layout_with_umap}
\alias{layout_igraph_umap}
\title{UMAP graph layouts}
\usage{
layout_with_umap(g, pivots = NULL, ...)

layout_igraph_umap(g, circular, ...)
}
\arguments{
\item{g}{igraph object}

\item{pivots}{if not NULL, number of pivot nodes to use for distance calculation (for large graphs).}

\item{...}{additional parameters for umap. See the \code{?uwot::umap} for help.}

\item{circular}{not used}
}
\value{
matrix of xy coordinates
}
\description{
Using the UMAP dimensionality reduction algorithm as a graph layout
}
\details{
The layout_igraph_* function should not be used directly. It is only used as an argument for plotting with 'igraph'. UMAP can be tuned by many different parameters. Refer to the documentation at https://github.com/jlmelville/uwot for help
}
\examples{
library(igraph)

g <- sample_islands(10, 20, 0.6, 10)
# xy <- layout_with_umap(g, min_dist = 0.5)
}
\references{
McInnes, Leland, John Healy, and James Melville. "Umap: Uniform manifold approximation and projection for dimension reduction." arXiv preprint arXiv:1802.03426 (2018).
}
\author{
David Schoch
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_stress.R, R/layouts.R
\name{layout_fixed_coords}
\alias{layout_fixed_coords}
\alias{layout_with_fixed_coords}
\alias{layout_igraph_fixed_coords}
\title{Layout with fixed coordinates}
\usage{
layout_with_fixed_coords(
  g,
  coords,
  weights = NA,
  iter = 500,
  tol = 1e-04,
  mds = TRUE,
  bbox = 30
)

layout_igraph_fixed_coords(
  g,
  coords,
  weights = NA,
  iter = 500,
  tol = 1e-04,
  mds = TRUE,
  bbox = 30,
  circular
)
}
\arguments{
\item{g}{igraph object}

\item{coords}{numeric n x 2 matrix, where n is the number of nodes. values
are either NA or fixed coordinates. coordinates are only calculated for the
NA values.}

\item{weights}{possibly a numeric vector with edge weights. If this is NULL and the graph has a weight edge attribute, then the attribute is used. If this is NA then no weights are used (even if the graph has a weight attribute). By default, weights are ignored. See details for more.}

\item{iter}{number of iterations during stress optimization}

\item{tol}{stopping criterion for stress optimization}

\item{mds}{should an MDS layout be used as initial layout (default: TRUE)}

\item{bbox}{constrain dimension of output. Only relevant to determine the placement of disconnected graphs}

\item{circular}{not used}
}
\value{
matrix of xy coordinates
}
\description{
force-directed graph layout based on stress majorization with
fixed coordinates for some nodes
}
\details{
Be careful when using weights. In most cases, the inverse of the edge weights should be used to ensure that the endpoints of an edges with higher weights are closer together (weights=1/E(g)$weight).

The layout_igraph_* function should not be used directly. It is only used as an argument for plotting with 'igraph'.
'ggraph' natively supports the layout.
}
\examples{
library(igraph)
set.seed(12)
g <- sample_bipartite(10, 5, "gnp", 0.5)
fxy <- cbind(c(rep(0, 10), rep(1, 5)), NA)
xy <- layout_with_fixed_coords(g, fxy)
}
\seealso{
\link{layout_constrained_stress}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_multilevel.R, R/layouts.R
\name{layout_multilevel}
\alias{layout_multilevel}
\alias{layout_as_multilevel}
\alias{layout_igraph_multilevel}
\title{multilevel layout}
\usage{
layout_as_multilevel(
  g,
  type = "all",
  FUN1,
  FUN2,
  params1 = NULL,
  params2 = NULL,
  ignore_iso = TRUE,
  project2D = TRUE,
  alpha = 35,
  beta = 45
)

layout_igraph_multilevel(
  g,
  type = "all",
  FUN1,
  FUN2,
  params1 = NULL,
  params2 = NULL,
  ignore_iso = TRUE,
  alpha = 35,
  beta = 45,
  circular
)
}
\arguments{
\item{g}{igraph object. Must have a vertex attribute "lvl" which is 1 or 2.}

\item{type}{one of "all", "separate","fix1" or "fix2". see details}

\item{FUN1}{if type="separate", the layout function to be used for level 1}

\item{FUN2}{if type="separate", the layout function to be used for level 2}

\item{params1}{named list of parameters for FUN1}

\item{params2}{named list of parameters for FUN2}

\item{ignore_iso}{treatment of isolates within levels. see details}

\item{project2D}{logical. Defaults to TRUE (project to 2D).}

\item{alpha}{angle for isometric projection between 0 and 90}

\item{beta}{angle for isometric projection between 0 and 90}

\item{circular}{not used}
}
\value{
matrix of xy coordinates
}
\description{
Layout algorithm to visualize multilevel networks
}
\details{
The algorithm internally computes a 3D layout where each level is in a separate y-plane.
The layout is then projected into 2D via an isometric mapping, controlled by the parameters
\code{alpha} and \code{beta}. It may take some adjusting to \code{alpha} and \code{beta} to find a good perspective.

If type="all", the layout is computed at once for the complete network.
For type="separate", two user specified layout algorithms (\code{FUN1} and \code{FUN2}) are used for the levels.
The named lists \code{param1} and \code{param2} can be used to set parameters for \code{FUN1} and \code{FUN2}.
This option helpful for situations where different structural features of the levels should be emphasized.

For type="fix1" and type="fix2" only one of the level layouts is fixed. The other one is calculated by optimizing the
inter level ties, such that they are drawn (almost) vertical.

The \code{ignore_iso} parameter controls the handling of isolates. If TRUE, nodes without inter level edges are ignored during the layout process
and added at the end. If FALSE they are left unchanged

The layout_igraph_* function should not be used directly. It is only used as an argument for plotting with 'igraph'.
}
\examples{
library(igraph)
data("multilvl_ex")
\dontrun{
# compute a layout for the whole network
xy <- layout_as_multilevel(multilvl_ex, type = "all", alpha = 25, beta = 45)

# compute a layout for each level separately and combine them
xy <- layout_as_multilevel(multilvl_ex,
    type = "separate",
    FUN1 = layout_as_backbone,
    FUN2 = layout_with_stress,
    alpha = 25, beta = 45
)
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_tree.R, R/layouts.R
\name{layout_tree_unrooted}
\alias{layout_tree_unrooted}
\alias{layout_as_tree_unrooted}
\alias{layout_igraph_tree_unrooted}
\title{unrooted tree layout}
\usage{
layout_as_tree_unrooted(
  g,
  mode = c("equalangle", "equaldaylight", "stress"),
  weights = NA,
  daylight_iter = 15,
  iter = 500,
  tol = 1e-04,
  bbox = 30
)

layout_igraph_tree_unrooted(
  g,
  mode = c("equalangle", "equaldaylight", "stress"),
  weights = NA,
  daylight_iter = 15,
  iter = 500,
  tol = 1e-04,
  bbox = 30,
  circular
)
}
\arguments{
\item{g}{igraph object. Each connected component must be a tree (acyclic).}

\item{mode}{which algorithm to use. One of \code{"equalangle"} (default),
\code{"equaldaylight"} or \code{"stress"}. See details.}

\item{weights}{possibly a numeric vector with edge weights, interpreted as
branch lengths. If this is NULL and the graph has a weight edge attribute,
then the attribute is used. If this is NA then unit branch lengths are used
(even if the graph has a weight attribute). By default, unit branch lengths
are used.}

\item{daylight_iter}{number of refinement sweeps for \code{mode = "equaldaylight"}.}

\item{iter}{number of iterations during stress optimization (only used for
\code{mode = "stress"}).}

\item{tol}{stopping criterion for stress optimization (only used for
\code{mode = "stress"}).}

\item{bbox}{width of layout. Only relevant to determine the placement of
disconnected components (forests).}

\item{circular}{not used}
}
\value{
matrix of xy coordinates
}
\description{
arranges the nodes of a tree (or forest) in the plane without a
designated root, using algorithms developed for drawing phylogenies.
}
\details{
Three algorithms are available:
\itemize{
\item \code{"equalangle"} (Felsenstein, 1989): each subtree is allotted an angular
wedge proportional to its number of leaves. Linear time, but can look
crowded on unbalanced trees.
\item \code{"equaldaylight"} (Felsenstein): iteratively equalizes the angular gaps
("daylight") around each internal node, starting from the equal-angle
layout, for a more balanced drawing.
\item \code{"stress"}: stress majorization using the patristic (tree path-length)
distances as target distances, reusing the package's stress engine.
}

Branch lengths (edge weights) are respected when present; otherwise unit
lengths are used.

The layout_igraph_* function should not be used directly. It is only used as
an argument for plotting with 'igraph'. 'ggraph' natively supports the layout.
}
\examples{
library(igraph)
g <- make_tree(20, 3, mode = "undirected")

xy <- layout_as_tree_unrooted(g)
xy <- layout_as_tree_unrooted(g, mode = "equaldaylight")
}
\references{
Felsenstein, J. (1989). PHYLIP - Phylogeny Inference Package
(Version 3.2). \emph{Cladistics}, 5, 164-166.

Bachmaier, C., Brandes, U., & Schlieper, B. (2005). Drawing phylogenetic
trees. \emph{In International Symposium on Algorithms and Computation} (pp.
1110-1121). Springer, Berlin, Heidelberg.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_stress.R
\name{layout_constrained_stress3D}
\alias{layout_constrained_stress3D}
\alias{layout_with_constrained_stress3D}
\title{constrained stress layout in 3D}
\usage{
layout_with_constrained_stress3D(
  g,
  coord,
  fixdim = "x",
  weights = NA,
  iter = 500,
  tol = 1e-04,
  mds = TRUE,
  bbox = 30
)
}
\arguments{
\item{g}{igraph object}

\item{coord}{numeric vector. fixed coordinates for dimension specified in \code{fixdim}.}

\item{fixdim}{string. which dimension should be fixed. Either "x", "y" or "z".}

\item{weights}{possibly a numeric vector with edge weights. If this is NULL and the graph has a weight edge attribute, then the attribute is used. If this is NA then no weights are used (even if the graph has a weight attribute). By default, weights are ignored. See details for more.}

\item{iter}{number of iterations during stress optimization}

\item{tol}{stopping criterion for stress optimization}

\item{mds}{should an MDS layout be used as initial layout (default: TRUE)}

\item{bbox}{constrain dimension of output. Only relevant to determine the placement of disconnected graphs}
}
\value{
matrix of xyz coordinates
}
\description{
force-directed graph layout based on stress majorization with variable constrained in 3D
}
\details{
Be careful when using weights. In most cases, the inverse of the edge weights should be used to ensure that the endpoints of an edges with higher weights are closer together (weights=1/E(g)$weight).

This function does not come with direct support for igraph or ggraph.
}
\references{
Gansner, E. R., Koren, Y., & North, S. (2004). Graph drawing by stress majorization. \emph{In International Symposium on Graph Drawing} (pp. 239-250). Springer, Berlin, Heidelberg.
}
\seealso{
\link{layout_constrained_stress}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-examples.R
\docType{data}
\name{multilvl_ex}
\alias{multilvl_ex}
\title{Multilevel example Network}
\format{
igraph object
}
\usage{
multilvl_ex
}
\description{
Multilevel network, where both levels have different structural features
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotate_functions.R
\name{annotate_circle}
\alias{annotate_circle}
\title{annotate concentric circles}
\usage{
annotate_circle(cent, col = "#00BFFF", format = "", pos = "top", text_size = 3)
}
\arguments{
\item{cent}{centrality scores used for layout}

\item{col}{color of text}

\item{format}{either empty string or 'scientific'}

\item{pos}{position of text ('top' or 'bottom')}

\item{text_size}{font size for annotations}
}
\value{
annotated concentric circles around origin
}
\description{
annotate concentric circles
}
\details{
this function is best used with \link{layout_with_centrality} together with \link{draw_circle}.
}
\examples{
library(igraph)

g <- sample_gnp(10, 0.4)
\dontrun{
library(ggraph)
ggraph(g, layout = "centrality", centrality = closeness(g)) +
    draw_circle(use = "cent") +
    annotate_circle(closeness(g), pos = "bottom", format = "scientific") +
    geom_edge_link() +
    geom_node_point(shape = 21, fill = "grey25", size = 5) +
    theme_graph() +
    coord_fixed()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_manipulate.R
\name{layout_manipulate}
\alias{layout_manipulate}
\alias{layout_rotate}
\alias{layout_mirror}
\title{manipulate layout}
\usage{
layout_rotate(xy, angle)

layout_mirror(xy, axis = "vertical")
}
\arguments{
\item{xy}{graph layout}

\item{angle}{angle for rotation}

\item{axis}{mirror horizontal or vertical}
}
\value{
manipulated matrix of xy coordinates
}
\description{
functions to manipulate an existing layout
}
\details{
These functions are mostly useful for deterministic layouts such as \link{layout_with_stress}
}
\examples{
library(igraph)
g <- sample_gnp(50, 0.3)

xy <- layout_with_stress(g)

# rotate 90 degrees
xy <- layout_rotate(xy, 90)

# flip horizontally
xy <- layout_mirror(xy, "horizontal")

}
\author{
David Schoch
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data-examples.R
\docType{data}
\name{metro_berlin}
\alias{metro_berlin}
\title{Subway network of Berlin}
\format{
igraph object
}
\usage{
metro_berlin
}
\description{
A dataset containing the subway network of Berlin
}
\references{
Kujala, Rainer, et al. "A collection of public transport network data sets for 25 cities." Scientific data 5 (2018): 180089.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_large_graphs.R, R/layouts.R
\name{layout_pmds}
\alias{layout_pmds}
\alias{layout_with_pmds}
\alias{layout_igraph_pmds}
\title{pivot MDS graph layout}
\usage{
layout_with_pmds(g, pivots, weights = NA, D = NULL, dim = 2)

layout_igraph_pmds(g, pivots, weights = NA, D = NULL, circular)
}
\arguments{
\item{g}{igraph object}

\item{pivots}{number of pivots}

\item{weights}{possibly a numeric vector with edge weights. If this is NULL and the graph has a weight edge attribute, then the attribute is used. If this is NA then no weights are used (even if the graph has a weight attribute). By default, weights are ignored. See details for more.}

\item{D}{precomputed distances from pivots to all nodes (if available, default: NULL)}

\item{dim}{dimensionality of layout (defaults to 2)}

\item{circular}{not used}
}
\value{
matrix of coordinates
}
\description{
similar to \link[igraph]{layout_with_mds} but uses only a small set of pivots for MDS. Considerably faster than MDS and thus applicable for larger graphs.
}
\details{
Be careful when using weights. In most cases, the inverse of the edge weights should be used to ensure that the endpoints of an edges with higher weights are closer together (weights=1/E(g)$weight)

The layout_igraph_* function should not be used directly. It is only used as an argument for plotting with 'igraph'.
'ggraph' natively supports the layout.
}
\examples{
\dontrun{
library(igraph)
library(ggraph)

g <- sample_gnp(1000, 0.01)

xy <- layout_with_pmds(g, pivots = 100)
}
}
\references{
Brandes, U. and Pich, C. (2006). Eigensolver Methods for Progressive Multidimensional Scaling of Large Data. In \emph{International Symposium on Graph Drawing} (pp. 42-53). Springer
}
\author{
David Schoch
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_stress.R, R/layouts.R
\name{layout_focus_group}
\alias{layout_focus_group}
\alias{layout_with_focus_group}
\alias{layout_igraph_focus_group}
\title{radial focus group layout}
\usage{
layout_with_focus_group(
  g,
  v,
  group,
  shrink = 10,
  weights = NA,
  iter = 500,
  tol = 1e-04
)

layout_igraph_focus_group(
  g,
  v,
  group,
  shrink = 10,
  weights = NA,
  iter = 500,
  tol = 1e-04,
  circular
)
}
\arguments{
\item{g}{igraph object}

\item{v}{id of focal node to be placed in the center}

\item{group}{vector indicating grouping of nodes}

\item{shrink}{shrink the reserved angle range for a group to increase the gaps between groups}

\item{weights}{possibly a numeric vector with edge weights. If this is NULL and the graph has a weight edge attribute, then the attribute is used. If this is NA then no weights are used (even if the graph has a weight attribute). By default, weights are ignored. See details for more.}

\item{iter}{number of iterations during stress optimization}

\item{tol}{stopping criterion for stress optimization}

\item{circular}{not used}
}
\value{
matrix of xy coordinates
}
\description{
arrange nodes in concentric circles around a focal node according to their distance from the focus and keep predefined groups in the same angle range.
}
\details{
Be careful when using weights. In most cases, the inverse of the edge weights should be used to ensure that the endpoints of an edges with higher weights are closer together (weights=1/E(g)$weight).
}
\examples{
library(igraph)
g <- sample_islands(4, 5, 0.8, 2)
grp <- as.character(rep(1:4, each = 5))
layout_with_focus_group(g, v = 1, group = grp, shrink = 10)
}
\seealso{
\link{layout_focus}
The layout_igraph_* function should not be used directly. It is only used as an argument for plotting with 'igraph'.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_stress.R, R/layouts.R
\name{layout_centrality_group}
\alias{layout_centrality_group}
\alias{layout_with_centrality_group}
\alias{layout_igraph_centrality_group}
\title{radial centrality group layout}
\usage{
layout_with_centrality_group(g, cent, group, shrink = 10, ...)

layout_igraph_centrality_group(g, cent, group, shrink = 10, circular, ...)
}
\arguments{
\item{g}{igraph object}

\item{cent}{centrality scores}

\item{group}{vector indicating grouping of nodes}

\item{shrink}{shrink the reserved angle range for a group to increase the gaps between groups}

\item{...}{additional arguments to layout_with_centrality
The layout_igraph_* function should not be used directly. It is only used as an argument for plotting with 'igraph'.
'ggraph' natively supports the layout.}

\item{circular}{not used}
}
\value{
matrix of xy coordinates
}
\description{
arranges nodes in concentric circles according to a centrality index and keeping groups within a angle range
}
\examples{
library(igraph)
}
\seealso{
\link{layout_centrality}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_stress.R, R/layouts.R
\name{layout_centrality}
\alias{layout_centrality}
\alias{layout_with_centrality}
\alias{layout_igraph_centrality}
\title{radial centrality layout}
\usage{
layout_with_centrality(
  g,
  cent,
  scale = TRUE,
  iter = 500,
  tol = 1e-04,
  tseq = seq(0, 1, 0.2)
)

layout_igraph_centrality(
  g,
  cent,
  scale = TRUE,
  iter = 500,
  tol = 1e-04,
  tseq = seq(0, 1, 0.2),
  circular
)
}
\arguments{
\item{g}{igraph object}

\item{cent}{centrality scores}

\item{scale}{logical. should centrality scores be scaled to \eqn{[0,100]}? (Default: TRUE)}

\item{iter}{number of iterations during stress optimization}

\item{tol}{stopping criterion for stress optimization}

\item{tseq}{numeric vector. increasing sequence of coefficients to combine regular stress and constraint stress. See details.}

\item{circular}{not used}
}
\value{
matrix of xy coordinates
}
\description{
arranges nodes in concentric circles according to a centrality index.
}
\details{
The function optimizes a convex combination of regular stress and a constrained stress function which forces
nodes to be arranged on concentric circles. The vector \code{tseq} is the sequence of parameters used for the convex combination.
In iteration i of the algorithm \eqn{tseq[i]} is used to combine regular and constraint stress as \eqn{(1-tseq[i])*stress_{regular}+tseq[i]*stress_{constraint}}. The sequence must be increasing, start at zero and end at one. The default setting should be a good choice for most graphs.

The layout_igraph_* function should not be used directly. It is only used as an argument for plotting with 'igraph'.
'ggraph' natively supports the layout.
}
\examples{
library(igraph)

g <- sample_gnp(10, 0.4)
\dontrun{
library(ggraph)
ggraph(g, layout = "centrality", centrality = closeness(g)) +
    draw_circle(use = "cent") +
    geom_edge_link0() +
    geom_node_point(shape = 21, fill = "grey25", size = 5) +
    theme_graph() +
    coord_fixed()
}
}
\references{
Brandes, U., & Pich, C. (2011). More flexible radial layout. Journal of Graph Algorithms and Applications, 15(1), 157-173.
}
\seealso{
\link{layout_centrality_group}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/metro_multicriteria.R
\name{layout_as_metromap}
\alias{layout_as_metromap}
\title{Metro Map Layout}
\usage{
layout_as_metromap(object, xy, l = 2, gr = 0.0025, w = rep(1, 5), bsize = 5)
}
\arguments{
\item{object}{original graph}

\item{xy}{initial layout of the original graph}

\item{l}{desired multiple of grid point spacing. (l*gr determines desired edge length)}

\item{gr}{grid spacing. (l*gr determines desired edge length)}

\item{w}{weight vector for criteria (see details)}

\item{bsize}{number of grid points a station can move away rom its original position}
}
\value{
new coordinates for stations
}
\description{
Metro map layout based on multicriteria optimization
}
\details{
The function optimizes the following five criteria using a hill climbing algorithm:
\itemize{
\item \emph{Angular Resolution Criterion}: The angles of incident edges at each station should be maximized, because if there is only a small angle between any two adjacent edges, then it can become difficult to distinguish between them
\item \emph{Edge Length Criterion}: The edge lengths across the whole map should be approximately equal to ensure regular spacing between stations. It is based on the preferred multiple, l, of the grid spacing, g. The purpose of the criterion is to penalize edges that are longer than or shorter than lg.
\item \emph{Balanced Edge Length Criterion}: The length of edges incident to a particular station should be similar
\item \emph{Line Straightness Criterion}: (not yet implemented) Edges that form part of a line should, where possible, be co-linear either side of each station that the line passes through
\item \emph{Octilinearity Criterion}: Each edge should be drawn horizontally, vertically, or diagonally at 45 degree, so we penalize edges that are not at a desired angle
}
}
\examples{
# the algorithm has problems with parallel edges
library(igraph)
g <- simplify(metro_berlin)
xy <- cbind(V(g)$lon, V(g)$lat) * 100

# the algorithm is not very stable. try playing with the parameters
\dontrun{
xy_new <- layout_as_metromap(g, xy, l = 2, gr = 0.5, w = c(100, 100, 1, 1, 100), bsize = 35)
}
}
\references{
Stott, Jonathan, et al. "Automatic metro map layout using multicriteria optimization." IEEE Transactions on Visualization and Computer Graphics 17.1 (2010): 101-114.
}
\author{
David Schoch
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/annotate_functions.R
\name{draw_circle}
\alias{draw_circle}
\title{Draw concentric circles}
\usage{
draw_circle(col = "#00BFFF", use = "focus", max.circle)
}
\arguments{
\item{col}{color of circles}

\item{use}{one of 'focus' or 'cent'}

\item{max.circle}{if use = 'focus' specifies the number of circles to draw}
}
\value{
concentric circles around origin
}
\description{
Draw concentric circles
}
\details{
this function is best used with a concentric layout such as \link{layout_with_focus} and \link{layout_with_centrality}.
}
\examples{
library(igraph)
g <- sample_gnp(10, 0.4)

\dontrun{
library(ggraph)
ggraph(g, layout = "centrality", centrality = degree(g)) +
    draw_circle(use = "cent") +
    geom_edge_link() +
    geom_node_point(shape = 21, fill = "grey25", size = 5) +
    theme_graph() +
    coord_fixed()
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_large_graphs.R, R/layouts.R
\name{layout_sparse_stress}
\alias{layout_sparse_stress}
\alias{layout_with_sparse_stress}
\alias{layout_igraph_sparse_stress}
\title{sparse stress graph layout}
\usage{
layout_with_sparse_stress(g, pivots, weights = NA, iter = 500)

layout_igraph_sparse_stress(g, pivots, weights = NA, iter = 500, circular)
}
\arguments{
\item{g}{igraph object}

\item{pivots}{number of pivots}

\item{weights}{ignored}

\item{iter}{number of iterations during stress optimization}

\item{circular}{not used}
}
\value{
matrix of xy coordinates
}
\description{
stress majorization for larger graphs based on a set of pivot nodes.
}
\details{
The layout_igraph_* function should not be used directly. It is only used as an argument for plotting with 'igraph'.
'ggraph' natively supports the layout.
}
\examples{
\dontrun{
library(igraph)
library(ggraph)

g <- sample_gnp(1000, 0.005)

ggraph(g, layout = "sparse_stress", pivots = 100) +
    geom_edge_link0(edge_colour = "grey66") +
    geom_node_point(shape = 21, fill = "grey25", size = 5) +
    theme_graph()
}
}
\references{
Ortmann, M. and Klimenta, M. and Brandes, U. (2016). A Sparse Stress Model. https://arxiv.org/pdf/1608.08909.pdf
}
\author{
David Schoch
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/graph_manipulate.R
\name{graph_manipulate}
\alias{graph_manipulate}
\alias{reorder_edges}
\title{Manipulate graph}
\usage{
reorder_edges(g, attr, desc = TRUE)
}
\arguments{
\item{g}{igraph object}

\item{attr}{edge attribute name used to sort edges}

\item{desc}{logical. sort in descending (default) or ascending order}
}
\value{
manipulated graph
}
\description{
functions to manipulate a graph
}
\details{
\code{reorder_edges()} allows to reorder edges according to an attribute so that edges are
drawn in the given order.
}
\examples{
library(igraph)

g <- sample_gnp(10, 0.5)
E(g)$attr <- 1:ecount(g)
gn <- reorder_edges(g,"attr")

}
\author{
David Schoch
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/layout_backbone.R, R/layouts.R
\name{layout_backbone}
\alias{layout_backbone}
\alias{layout_as_backbone}
\alias{layout_igraph_backbone}
\title{backbone graph layout}
\usage{
layout_as_backbone(g, keep = 0.2, backbone = TRUE)

layout_igraph_backbone(g, keep = 0.2, backbone = TRUE, circular)
}
\arguments{
\item{g}{igraph object}

\item{keep}{fraction of edges to keep during backbone calculation}

\item{backbone}{logical. Return edge ids of the backbone (Default: TRUE)}

\item{circular}{not used}
}
\value{
list of xy coordinates and vector of edge ids included in the backbone
}
\description{
emphasizes a hidden group structure if it exists in the graph. Calculates a layout for a sparsified network only including the most embedded edges. Deleted edges are added back after the layout is calculated.
}
\details{
The layout_igraph_* function should not be used directly. It is only used as an argument for plotting with 'igraph'.
'ggraph' natively supports the layout.
}
\examples{
library(igraph)
\dontrun{
g <- sample_islands(9, 20, 0.4, 9)
g <- simplify(g)
V(g)$grp <- as.character(rep(1:9, each = 20))
bb <- layout_as_backbone(g, keep = 0.4)

# add backbone links as edge attribute
E(g)$col <- FALSE
E(g)$col[bb$backbone] <- TRUE
}
}
\references{
Nocaj, A., Ortmann, M., & Brandes, U. (2015). Untangling the hairballs of multi-centered, small-world online social media networks. Journal of Graph Algorithms and Applications: JGAA, 19(2), 595-618.
}
